@@ -15,26 +15,29 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake. If not, see <http://www.gnu.org/licenses/>.
-
+from __future__ import division
 import time
+import math
 import os.path
 import operator
 import itertools
 import logging
 import collections
-import mock
 import numpy
 
 from openquake.baselib import hdf5
 from openquake.baselib.python3compat import zip
-from openquake.baselib.general import AccumDict, block_splitter, humansize
+from openquake.baselib.general import (
+    AccumDict, block_splitter, humansize, split_in_slices)
 from openquake.hazardlib.calc.filters import FarAwayRupture, SourceFilter
+from openquake.hazardlib.gsim.base import ContextMaker
 from openquake.hazardlib.probability_map import ProbabilityMap
 from openquake.hazardlib.stats import compute_pmap_stats
-from openquake.risklib.riskinput import GmfGetter, str2rsi, rsi2str, indices_dt
+from openquake.risklib.riskinput import str2rsi, rsi2str, indices_dt
 from openquake.baselib import parallel
 from openquake.commonlib import calc, util, readinput
 from openquake.calculators import base
+from openquake.calculators.getters import GmfGetter, EBRupture, RuptureGetter
 from openquake.calculators.classical import (
     ClassicalCalculator, saving_sources_by_task)
 
@@ -90,9 +93,10 @@ def compute_ruptures(sources, src_filter, gsims, param, monitor):
     grp_id = sources[0].src_group_id
     eb_ruptures = []
     calc_times = []
-    rup_mon = monitor('filtering ruptures', measuremem=False)
+    rup_mon = monitor('making contexts', measuremem=False)
     # Compute and save stochastic event sets
     num_ruptures = 0
+    cmaker = ContextMaker(gsims, src_filter.integration_distance)
     for src, s_sites in src_filter(sources):
         t0 = time.time()
         if s_sites is None:
@@ -105,8 +109,7 @@ def compute_ruptures(sources, src_filter, gsims, param, monitor):
         # more efficient to filter only the ruptures that occur, i.e.
         # to call sample_ruptures *before* the filtering
         for ebr in _build_eb_ruptures(
-                src, num_occ_by_rup, src_filter.integration_distance,
-                s_sites, param['seed'], rup_mon):
+                src, num_occ_by_rup, cmaker, s_sites, param['seed'], rup_mon):
             eb_ruptures.append(ebr)
         dt = time.time() - t0
         calc_times.append((src.id, dt))
@@ -145,7 +148,7 @@ def sample_ruptures(src, num_ses, num_samples, seed):
 
 
 def _build_eb_ruptures(
-        src, num_occ_by_rup, idist, s_sites, random_seed, rup_mon):
+        src, num_occ_by_rup, cmaker, s_sites, random_seed, rup_mon):
     """
     Filter the ruptures stored in the dictionary num_occ_by_rup and
     yield pairs (rupture, <list of associated EBRuptures>)
@@ -153,7 +156,8 @@ def _build_eb_ruptures(
     for rup in sorted(num_occ_by_rup, key=operator.attrgetter('rup_no')):
         with rup_mon:
             try:
-                r_sites, dists = idist.get_closest(s_sites, rup)
+                rup.ctx = cmaker.make_contexts(s_sites, rup)
+                indices = rup.ctx[0].sids
             except FarAwayRupture:
                 # ignore ruptures which are far away
                 del num_occ_by_rup[rup]  # save memory
@@ -167,16 +171,10 @@ def _build_eb_ruptures(
             for _ in range(num_occ):
                 # NB: the 0 below is a placeholder; the right eid will be
                 # set a bit later, in set_eids
-                events.append((0, ses_idx, sampleid))
+                events.append((0, src.src_group_id, ses_idx, sampleid))
         if events:
-            yield calc.EBRupture(
-                rup, r_sites.indices,
-                numpy.array(events, calc.event_dt),
-                src.src_group_id, serial)
-
-
-def _count(ruptures):
-    return sum(ebr.multiplicity for ebr in ruptures)
+            yield EBRupture(rup, indices, numpy.array(events, calc.event_dt),
+                            serial)
 
 
 def get_events(ebruptures):
@@ -207,8 +205,7 @@ class EventBasedRuptureCalculator(base.HazardCalculator):
         minimum_intensity dictionary.
         """
         oq = self.oqparam
-        self.min_iml = calc.fix_minimum_intensity(
-            oq.minimum_intensity, oq.imtls)
+        self.min_iml = self.get_min_iml(oq)
         self.rupser = calc.RuptureSerializer(self.datastore)
 
     def zerodict(self):
@@ -218,7 +215,7 @@ class EventBasedRuptureCalculator(base.HazardCalculator):
         zd = AccumDict()
         zd.calc_times = []
         zd.eff_ruptures = AccumDict()
-        self.grp_trt = self.csm.info.grp_trt()
+        self.grp_trt = self.csm.info.grp_by("trt")
         return zd
 
     def agg_dicts(self, acc, ruptures_by_grp_id):
@@ -261,10 +258,15 @@ class EventBasedRuptureCalculator(base.HazardCalculator):
         :yields: (sources, sites, gsims, monitor) tuples
         """
         oq = self.oqparam
-        csm = self.csm  # we may filter here in the future
-        maxweight = csm.get_maxweight(oq.concurrent_tasks)
-        numheavy = len(csm.get_sources('heavy', maxweight))
-        logging.info('Using maxweight=%d, numheavy=%d', maxweight, numheavy)
+        src_filter = SourceFilter(self.sitecol, oq.maximum_distance)
+        csm = self.csm.filter(src_filter)
+
+        def weight(src):
+            return src.num_ruptures * src.RUPTURE_WEIGHT
+        maxweight = math.ceil(
+            sum(weight(src) for src in self.csm.get_sources()) /
+            (oq.concurrent_tasks or 1))
+        logging.info('Using maxweight=%d', maxweight)
         param = dict(
             truncation_level=oq.truncation_level,
             imtls=oq.imtls, seed=oq.ses_seed,
@@ -277,9 +279,10 @@ class EventBasedRuptureCalculator(base.HazardCalculator):
             for sg in sm.src_groups:
                 gsims = csm.info.gsim_lt.get_gsims(sg.trt)
                 csm.add_infos(sg.sources)
-                for block in csm.split_in_blocks(maxweight, sg.sources):
+                for block in csm.split_in_blocks(
+                        maxweight, sg.sources, weight):
                     block.samples = sm.samples
-                    yield block, csm.src_filter, gsims, param, monitor
+                    yield block, src_filter, gsims, param, monitor
                     num_tasks += 1
                     num_sources += len(block)
         logging.info('Sent %d sources in %d tasks', num_sources, num_tasks)
@@ -288,7 +291,7 @@ class EventBasedRuptureCalculator(base.HazardCalculator):
         mutex_groups = list(self.csm.gen_mutex_groups())
         assert not mutex_groups, 'Mutex sources are not implemented!'
         with self.monitor('managing sources', autoflush=True):
-            allargs = self.gen_args(self.csm, self.monitor('pmap_from_trt'))
+            allargs = self.gen_args(self.csm, self.monitor('classical'))
             iterargs = saving_sources_by_task(allargs, self.datastore)
             if isinstance(allargs, list):
                 # there is a trick here: if the arguments are known
@@ -307,107 +310,104 @@ class EventBasedRuptureCalculator(base.HazardCalculator):
         Save the SES collection
         """
         self.rupser.close()
-        num_events = sum(_count(ruptures) for ruptures in result.values())
-        num_ruptures = sum(len(ruptures) for ruptures in result.values())
+        num_events = sum(set_counts(self.datastore, 'events').values())
         if num_events == 0:
             raise RuntimeError(
                 'No seismic events! Perhaps the investigation time is too '
                 'small or the maximum_distance is too small')
+        num_ruptures = sum(len(ruptures) for ruptures in result.values())
         logging.info('Setting %d event years on %d ruptures',
                      num_events, num_ruptures)
         with self.monitor('setting event years', measuremem=True,
                           autoflush=True):
             numpy.random.seed(self.oqparam.ses_seed)
-            set_random_years(self.datastore,
+            set_random_years(self.datastore, 'events',
                              int(self.oqparam.investigation_time))
 
 
-def set_random_years(dstore, investigation_time):
+def set_counts(dstore, dsetname):
     """
-    Sort the `events` array and attach year labels sensitive to the
+    :param dstore: a DataStore instance
+    :dsetname: name of dataset with a field `grp_id`
+    :returns: a dictionary grp_id > counts
+    """
+    groups = dstore[dsetname]['grp_id']
+    unique, counts = numpy.unique(groups, return_counts=True)
+    dic = dict(zip(unique, counts))
+    dstore.set_attrs(dsetname, by_grp=sorted(dic.items()))
+    return dic
+
+
+def set_random_years(dstore, name, investigation_time):
+    """
+    Set on the `events` dataset year labels sensitive to the
     SES ordinal and the investigation time.
     """
-    events = dstore['events'].value
-    eids = numpy.sort(events['eid'])
+    events = dstore[name].value
     years = numpy.random.choice(investigation_time, len(events)) + 1
-    year_of = dict(zip(eids, years))
+    year_of = dict(zip(numpy.sort(events['eid']), years))  # eid -> year
     for event in events:
         idx = event['ses'] - 1  # starts from 0
         event['year'] = idx * investigation_time + year_of[event['eid']]
-    dstore['events'] = events
+    dstore[name] = events
 
 
 # ######################## GMF calculator ############################ #
 
 
-def compute_gmfs_and_curves(getter, oq, monitor):
+def compute_gmfs_and_curves(getters, oq, monitor):
     """
-    :param getter:
-        a GmfGetter instance
+    :param getters:
+        a list of GmfGetter instances
     :param oq:
         an OqParam instance
     :param monitor:
         a Monitor instance
     :returns:
-        a dictionary with keys gmfcoll and hcurves
-   """
-    with monitor('making contexts', measuremem=True):
-        getter.init()
-    hcurves = {}  # key -> poes
-    if oq.hazard_curves_from_gmfs:
-        hc_mon = monitor('building hazard curves', measuremem=False)
-        duration = oq.investigation_time * oq.ses_per_logic_tree_path
-        with monitor('building hazard', measuremem=True):
-            gmfdata = numpy.fromiter(getter.gen_gmv(), getter.gmf_data_dt)
-            hazard = getter.get_hazard(data=gmfdata)
-        for sid, hazardr in zip(getter.sids, hazard):
-            for rlzi, array in hazardr.items():
-                if len(array) == 0:  # no data
-                    continue
-                with hc_mon:
-                    gmvs = array['gmv']
-                    for imti, imt in enumerate(getter.imtls):
-                        poes = calc._gmvs_to_haz_curve(
-                            gmvs[:, imti], oq.imtls[imt],
-                            oq.investigation_time, duration)
-                        hcurves[rsi2str(rlzi, sid, imt)] = poes
-    else:  # fast lane
-        with monitor('building hazard', measuremem=True):
-            gmfdata = numpy.fromiter(getter.gen_gmv(), getter.gmf_data_dt)
-    indices = []
-    if oq.ground_motion_fields:
-        gmfdata.sort(order=('sid', 'rlzi', 'eid'))
-        start = stop = 0
-        for sid, rows in itertools.groupby(gmfdata['sid']):
-            for row in rows:
-                stop += 1
-            indices.append((sid, start, stop))
-            start = stop
-    else:
-        gmfdata = None
-    return dict(gmfdata=gmfdata, hcurves=hcurves, gmdata=getter.gmdata,
-                taskno=monitor.task_no, indices=numpy.array(indices, (U32, 3)))
-
-
-def get_ruptures_by_grp(dstore):
+        a list of dictionaries with keys gmfcoll and hcurves
     """
-    Extracts the dictionary `ruptures_by_grp` from the given calculator
-    """
-    events = dstore['events']
-    n = 0
-    for grp in dstore['ruptures']:
-        n += len(dstore['ruptures/' + grp])
-    logging.info('Reading %d ruptures from the datastore', n)
-    # disable check on PlaceSurface to support UCERF ruptures
-    with mock.patch(
-            'openquake.hazardlib.geo.surface.PlanarSurface.'
-            'IMPERFECT_RECTANGLE_TOLERANCE', numpy.inf):
-        ruptures_by_grp = AccumDict(accum=[])
-        for grp in dstore['ruptures']:
-            grp_id = int(grp[4:])  # strip 'grp-'
-            ruptures = list(calc.get_ruptures(dstore, events, grp_id))
-            ruptures_by_grp[grp_id] = ruptures
-    return ruptures_by_grp
+    results = []
+    for getter in getters:
+        with monitor('GmfGetter.init', measuremem=True):
+            getter.init()
+        hcurves = {}  # key -> poes
+        if oq.hazard_curves_from_gmfs:
+            hc_mon = monitor('building hazard curves', measuremem=False)
+            duration = oq.investigation_time * oq.ses_per_logic_tree_path
+            with monitor('building hazard', measuremem=True):
+                gmfdata = numpy.fromiter(getter.gen_gmv(), getter.gmf_data_dt)
+                hazard = getter.get_hazard(data=gmfdata)
+            for sid, hazardr in zip(getter.sids, hazard):
+                for rlzi, array in hazardr.items():
+                    if len(array) == 0:  # no data
+                        continue
+                    with hc_mon:
+                        gmvs = array['gmv']
+                        for imti, imt in enumerate(getter.imtls):
+                            poes = calc._gmvs_to_haz_curve(
+                                gmvs[:, imti], oq.imtls[imt],
+                                oq.investigation_time, duration)
+                            hcurves[rsi2str(rlzi, sid, imt)] = poes
+        else:  # fast lane
+            with monitor('building hazard', measuremem=True):
+                gmfdata = numpy.fromiter(getter.gen_gmv(), getter.gmf_data_dt)
+        indices = []
+        if oq.ground_motion_fields:
+            gmfdata.sort(order=('sid', 'rlzi', 'eid'))
+            start = stop = 0
+            for sid, rows in itertools.groupby(gmfdata['sid']):
+                for row in rows:
+                    stop += 1
+                indices.append((sid, start, stop))
+                start = stop
+        else:
+            gmfdata = None
+        res = dict(gmfdata=gmfdata, hcurves=hcurves, gmdata=getter.gmdata,
+                   taskno=monitor.task_no,
+                   indices=numpy.array(indices, (U32, 3)))
+        if len(getter.gmdata):
+            results.append(res)
+    return results
 
 
 def save_gmdata(calc, n_rlzs):
@@ -447,70 +447,92 @@ class EventBasedCalculator(base.HazardCalculator):
     core_task = compute_gmfs_and_curves
     is_stochastic = True
 
-    def combine_pmaps_and_save_gmfs(self, acc, res):
+    def combine_pmaps_and_save_gmfs(self, acc, results):
         """
         Combine the hazard curves (if any) and save the gmfs (if any)
         sequentially; notice that the gmfs may come from
         different tasks in any order.
 
         :param acc: an accumulator for the hazard curves
-        :param res: a dictionary rlzi, imt -> [gmf_array, curves_by_imt]
+        :param results: dictionaries rlzi, imt -> [gmf_array, curves_by_imt]
         :returns: a new accumulator
         """
         sav_mon = self.monitor('saving gmfs')
         agg_mon = self.monitor('aggregating hcurves')
-        self.gmdata += res['gmdata']
-        data = res['gmfdata']
-        if data is not None:
-            with sav_mon:
-                hdf5.extend3(self.datastore.hdf5path, 'gmf_data/data', data)
-                # it is important to save the number of bytes while the
-                # computation is going, to see the progress
-                update_nbytes(self.datastore, 'gmf_data/data', data)
-                for sid, start, stop in res['indices']:
-                    self.indices[sid].append(
-                        (start + self.offset, stop + self.offset))
-                self.offset += len(data)
-        slicedic = self.oqparam.imtls.slicedic
-        with agg_mon:
-            for key, poes in res['hcurves'].items():
-                rlzi, sid, imt = str2rsi(key)
-                array = acc[rlzi].setdefault(sid, 0).array[slicedic[imt], 0]
-                array[:] = 1. - (1. - array) * (1. - poes)
-        sav_mon.flush()
-        agg_mon.flush()
-        self.datastore.flush()
-        if 'ruptures' in res:
-            vars(EventBasedRuptureCalculator)['save_ruptures'](
-                self, res['ruptures'])
+        hdf5path = self.datastore.hdf5path
+        for res in results:
+            self.gmdata += res['gmdata']
+            data = res['gmfdata']
+            if data is not None:
+                with sav_mon:
+                    hdf5.extend3(hdf5path, 'gmf_data/data', data)
+                    # it is important to save the number of bytes while the
+                    # computation is going, to see the progress
+                    update_nbytes(self.datastore, 'gmf_data/data', data)
+                    for sid, start, stop in res['indices']:
+                        self.indices[sid].append(
+                            (start + self.offset, stop + self.offset))
+                    self.offset += len(data)
+            slicedic = self.oqparam.imtls.slicedic
+            with agg_mon:
+                for key, poes in res['hcurves'].items():
+                    r, sid, imt = str2rsi(key)
+                    array = acc[r].setdefault(sid, 0).array[slicedic[imt], 0]
+                    array[:] = 1. - (1. - array) * (1. - poes)
+            sav_mon.flush()
+            agg_mon.flush()
+            self.datastore.flush()
+            if 'ruptures' in res:
+                vars(EventBasedRuptureCalculator)['save_ruptures'](
+                    self, res['ruptures'])
         return acc
 
-    def gen_args(self, ruptures_by_grp):
+    def gen_args(self):
         """
-        :param ruptures_by_grp: a dictionary of EBRupture objects
         :yields: the arguments for compute_gmfs_and_curves
         """
         oq = self.oqparam
         monitor = self.monitor(self.core_task.__name__)
         imts = list(oq.imtls)
-        min_iml = calc.fix_minimum_intensity(oq.minimum_intensity, imts)
+        min_iml = self.get_min_iml(oq)
         correl_model = oq.get_correl_model()
         try:
             csm_info = self.csm.info
         except AttributeError:  # no csm
             csm_info = self.datastore['csm_info']
         samples_by_grp = csm_info.get_samples_by_grp()
-        for grp_id in ruptures_by_grp:
-            ruptures = ruptures_by_grp[grp_id]
-            if not ruptures:
-                continue
-            rlzs_by_gsim = self.rlzs_assoc.get_rlzs_by_gsim(grp_id)
-            for block in block_splitter(ruptures, oq.ruptures_per_block):
-                samples = samples_by_grp[grp_id]
-                getter = GmfGetter(rlzs_by_gsim, block, self.sitecol,
-                                   imts, min_iml, oq.truncation_level,
-                                   correl_model, samples)
-                yield getter, oq, monitor
+        rlzs_by_gsim = {grp_id: self.rlzs_assoc.get_rlzs_by_gsim(grp_id)
+                        for grp_id in samples_by_grp}
+        if self.precalc:
+            num_ruptures = sum(len(rs) for rs in self.precalc.result.values())
+            block_size = math.ceil(num_ruptures / (oq.concurrent_tasks or 1))
+            for grp_id, ruptures in self.precalc.result.items():
+                if not ruptures:
+                    continue
+                for block in block_splitter(ruptures, block_size):
+                    getter = GmfGetter(
+                        rlzs_by_gsim[grp_id], block, self.sitecol,
+                        imts, min_iml, oq.maximum_distance,
+                        oq.truncation_level, correl_model,
+                        samples_by_grp[grp_id])
+                    yield [getter], oq, monitor
+            return
+        U = len(self.datastore['ruptures'])
+        logging.info('Found %d ruptures', U)
+        parent = self.can_read_parent() or self.datastore
+        for slc in split_in_slices(U, oq.concurrent_tasks or 1):
+            getters = []
+            for grp_id in rlzs_by_gsim:
+                ruptures = RuptureGetter(parent, slc, grp_id)
+                if parent is self.datastore:  # not accessible parent
+                    ruptures = list(ruptures)
+                    if not ruptures:
+                        continue
+                getters.append(GmfGetter(
+                    rlzs_by_gsim[grp_id], ruptures, self.sitecol,
+                    imts, min_iml, oq.maximum_distance, oq.truncation_level,
+                    correl_model, samples_by_grp[grp_id]))
+            yield getters, oq, monitor
 
     def execute(self):
         """
@@ -524,21 +546,17 @@ class EventBasedCalculator(base.HazardCalculator):
         if self.oqparam.ground_motion_fields:
             calc.check_overflow(self)
 
-        with self.monitor('reading ruptures', autoflush=True):
-            ruptures_by_grp = (self.precalc.result if self.precalc
-                               else get_ruptures_by_grp(self.datastore.parent))
-
         self.csm_info = self.datastore['csm_info']
         self.sm_id = {tuple(sm.path): sm.ordinal
                       for sm in self.csm_info.source_models}
         L = len(oq.imtls.array)
-        R = len(self.datastore['realizations'])
-        allargs = list(self.gen_args(ruptures_by_grp))
-        res = parallel.Starmap(self.core_task.__func__, allargs).submit_all()
+        R = self.datastore['csm_info'].get_num_rlzs()
         self.gmdata = {}
         self.offset = 0
         self.indices = collections.defaultdict(list)  # sid -> indices
-        acc = res.reduce(self.combine_pmaps_and_save_gmfs, {
+        acc = parallel.Starmap(
+            self.core_task.__func__, self.gen_args()
+        ).reduce(self.combine_pmaps_and_save_gmfs, {
             r: ProbabilityMap(L) for r in range(R)})
         save_gmdata(self, R)
         if self.indices:
@@ -568,7 +586,7 @@ class EventBasedCalculator(base.HazardCalculator):
         if not oq.hazard_curves_from_gmfs and not oq.ground_motion_fields:
             return
         elif oq.hazard_curves_from_gmfs:
-            rlzs = self.datastore['realizations'].value
+            rlzs = self.rlzs_assoc.realizations
             # save individual curves
             for i in sorted(result):
                 key = 'hcurves/rlz-%03d' % i
@@ -580,12 +598,14 @@ class EventBasedCalculator(base.HazardCalculator):
             # compute and save statistics; this is done in process
             # we don't need to parallelize, since event based calculations
             # involves a "small" number of sites (<= 65,536)
-            weights = [rlz['weight'] for rlz in rlzs]
+            weights = [rlz.weight for rlz in rlzs]
             hstats = self.oqparam.hazard_stats()
             if len(hstats) and len(rlzs) > 1:
                 for kind, stat in hstats:
                     pmap = compute_pmap_stats(result.values(), [stat], weights)
                     self.datastore['hcurves/' + kind] = pmap
+        if self.datastore.parent:
+            self.datastore.parent.open()
         if 'gmf_data' in self.datastore:
             self.save_gmf_bytes()
         if oq.compare_with_classical:  # compute classical curves
