@@ -25,11 +25,13 @@ from openquake.baselib import hdf5, parallel, performance
 from openquake.baselib.python3compat import decode, encode
 from openquake.baselib.general import (
     group_array, split_in_blocks, deprecated as depr)
+from openquake.hazardlib import nrml
 from openquake.hazardlib.stats import compute_stats2
-from openquake.risklib import scientific, riskinput
+from openquake.risklib import scientific
 from openquake.calculators.export import export, loss_curves
-from openquake.calculators.export.hazard import savez
-from openquake.commonlib import writers, calc
+from openquake.calculators.export.hazard import savez, get_mesh
+from openquake.calculators import getters
+from openquake.commonlib import writers, hazard_writers
 from openquake.commonlib.util import (
     get_assets, compose_arrays, reader)
 
@@ -95,7 +97,7 @@ def export_avg_losses(ekey, dstore):
     writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
     name, kind = dskey.split('-')
     if kind == 'stats':
-        weights = dstore['realizations']['weight']
+        weights = dstore['csm_info'].rlzs['weight']
         tags, stats = zip(*oq.risk_stats())
         if dskey in dstore:  # precomputed
             value = dstore[dskey].value
@@ -104,7 +106,8 @@ def export_avg_losses(ekey, dstore):
                 dstore['avg_losses-rlzs'].value, stats, weights)
     else:  # rlzs
         value = dstore[dskey].value  # shape (A, R, LI)
-        tags = ['rlz-%03d' % r for r in range(len(dstore['realizations']))]
+        R = value.shape[1]
+        tags = ['rlz-%03d' % r for r in range(R)]
     for tag, values in zip(tags, value.transpose(1, 0, 2)):
         dest = dstore.build_fname(name, tag, 'csv')
         array = numpy.zeros(len(values), dt)
@@ -199,6 +202,25 @@ def export_all_losses_npz(ekey, dstore):
     return [fname]
 
 
+@export.add(('rup_loss_table', 'xml'))
+def export_maxloss_ruptures(ekey, dstore):
+    """
+    :param ekey: export key, i.e. a pair (datastore key, fmt)
+    :param dstore: datastore object
+    """
+    oq = dstore['oqparam']
+    mesh = get_mesh(dstore['sitecol'])
+    fnames = []
+    for loss_type in oq.loss_dt().names:
+        ebr = getters.get_maxloss_rupture(dstore, loss_type)
+        root = hazard_writers.rupture_to_element(ebr.export(mesh))
+        dest = dstore.export_path('rupture-%s.xml' % loss_type)
+        with open(dest, 'wb') as fh:
+            nrml.write(list(root), fh)
+        fnames.append(dest)
+    return fnames
+
+
 # this is used by event_based_risk
 @export.add(('agg_loss_table', 'csv'))
 @depr('This exporter will be removed soon')
@@ -207,6 +229,9 @@ def export_agg_losses_ebr(ekey, dstore):
     :param ekey: export key, i.e. a pair (datastore key, fmt)
     :param dstore: datastore object
     """
+    if 'ruptures' not in dstore:
+        logging.warn('There are no ruptures in the datastore')
+        return []
     name, ext = export.keyfunc(ekey)
     agg_losses = dstore[name]
     has_rup_data = 'ruptures' in dstore
@@ -226,11 +251,13 @@ def export_agg_losses_ebr(ekey, dstore):
     rup_data = {}
     event_by_eid = {}  # eid -> event
     # populate rup_data and event_by_eid
+    ruptures_by_grp = getters.get_ruptures_by_grp(dstore)
+    # TODO: avoid reading the events twice
     for grp_id, events in all_events.items():
         for event in events:
             event_by_eid[event['eid']] = event
         if has_rup_data:
-            ruptures = calc.get_ruptures(dstore, the_events, grp_id)
+            ruptures = ruptures_by_grp.get(grp_id, [])
             rup_data.update(get_rup_data(ruptures))
     for r, row in enumerate(agg_losses):
         rec = elt[r]
@@ -264,7 +291,7 @@ def export_loss_curves(ekey, dstore):
 # this is used by classical_risk and event_based_risk
 @export.add(('loss_curves-stats', 'csv'))
 def export_loss_curves_stats(ekey, dstore):
-    num_rlzs = len(dstore['realizations'])
+    num_rlzs = dstore['csm_info'].get_num_rlzs()
     kind = 'stats' if num_rlzs > 1 else 'rlzs'
     return export_loss_curves(('loss_curves/' + kind, 'csv'), dstore)
 
@@ -293,7 +320,7 @@ def export_loss_maps_npz(ekey, dstore):
     kind = ekey[0].split('-')[1]  # rlzs or stats
     assets = get_assets(dstore)
     value = get_loss_maps(dstore, kind)
-    R = len(dstore['realizations'])
+    R = dstore['csm_info'].get_num_rlzs()
     if kind == 'rlzs':
         tags = ['rlz-%03d' % r for r in range(R)]
     else:
@@ -400,7 +427,7 @@ def export_dmg_by_asset_npz(ekey, dstore):
 @depr('This output will be removed soon')
 def export_dmg_by_tag_csv(ekey, dstore):
     damage_dt = build_damage_dt(dstore)
-    tags = add_quotes(dstore['assetcol'].tags())
+    tags = add_quotes(dstore['assetcol'].tagcol)
     rlzs = dstore['csm_info'].get_rlzs_assoc().realizations
     data = dstore[ekey[0]]
     writer = writers.CsvWriter(fmt='%.6E')
@@ -520,23 +547,21 @@ def export_bcr_map(ekey, dstore):
         tags = ['mean'] + ['quantile-%s' % q for q in oq.quantile_loss_curves]
     else:
         tags = ['rlz-%03d' % r for r in range(R)]
-    loss_types = dstore.get_attr('composite_risk_model', 'loss_types')
     fnames = []
     writer = writers.CsvWriter(fmt=writers.FIVEDIGITS)
     for t, tag in enumerate(tags):
-        for l, loss_type in enumerate(loss_types):
-            rlz_data = bcr_data[loss_type][:, t]
-            path = dstore.build_fname('bcr-%s' % loss_type, tag, 'csv')
-            data = [['lon', 'lat', 'asset_ref', 'average_annual_loss_original',
-                     'average_annual_loss_retrofitted', 'bcr']]
-            for ass, value in zip(assetcol, rlz_data):
-                data.append((ass['lon'], ass['lat'],
-                             decode(aref[ass['idx']]),
-                             value['annual_loss_orig'],
-                             value['annual_loss_retro'],
-                             value['bcr']))
-            writer.save(data, path)
-            fnames.append(path)
+        rlz_data = bcr_data[:, t]
+        path = dstore.build_fname('bcr', tag, 'csv')
+        data = [['lon', 'lat', 'asset_ref', 'average_annual_loss_original',
+                 'average_annual_loss_retrofitted', 'bcr']]
+        for ass, value in zip(assetcol, rlz_data):
+            data.append((ass['lon'], ass['lat'],
+                         decode(aref[ass['idx']]),
+                         value['annual_loss_orig'],
+                         value['annual_loss_retro'],
+                         value['bcr']))
+        writer.save(data, path)
+        fnames.append(path)
     return writer.getsaved()
 
 
@@ -574,7 +599,7 @@ def export_asset_loss_table(ekey, dstore):
     fname = dstore.export_path('%s.%s' % ekey)
     monitor = performance.Monitor(key, fname)
     aids = range(len(assetcol))
-    allargs = [(riskinput.LossRatiosGetter(dstore, block), monitor)
+    allargs = [(getters.LossRatiosGetter(dstore, block), monitor)
                for block in split_in_blocks(aids, oq.concurrent_tasks)]
     dstore.close()  # avoid OSError: Can't read data (Wrong b-tree signature)
     L = len(loss_types)
