@@ -34,9 +34,9 @@ from openquake.qa_tests_data.classical import (
 class ClassicalTestCase(CalculatorTestCase):
 
     def assert_curves_ok(self, expected, test_dir, delta=None, **kw):
+        kind = kw.pop('kind', '')  # 'all' or ''
         self.run_calc(test_dir, 'job.ini', **kw)
         ds = self.calc.datastore
-        kind = kw.get('kind', '')  # 'all' or ''
         got = (export(('hcurves/' + kind, 'csv'), ds) +
                export(('hmaps/' + kind, 'csv'), ds) +
                export(('uhs/' + kind, 'csv'), ds))
@@ -67,7 +67,7 @@ class ClassicalTestCase(CalculatorTestCase):
 
         # check extraction
         sitecol = extract(self.calc.datastore, 'sitecol')
-        self.assertEqual(repr(sitecol), '<SiteCollection with 1 sites>')
+        self.assertEqual(repr(sitecol), '<SiteCollection with 1/1 sites>')
 
     @attr('qa', 'hazard', 'classical')
     def test_wrong_smlt(self):
@@ -123,13 +123,10 @@ class ClassicalTestCase(CalculatorTestCase):
              'hazard_curve-smltp_b2-gsimltp_b1.csv'],
             case_7.__file__, kind='all')
 
-        with self.assertRaises(ValueError) as ctx:
-            self.run_calc(
-                case_7.__file__, 'job.ini', mean_hazard_curves='false',
-                hazard_maps='true', poes='0.1')
-        self.assertEqual(
-            'The job.ini says that no statistics should be computed, but then '
-            'there is no output!', str(ctx.exception))
+        # exercise the warning for no output when mean_hazard_curves='false'
+        self.run_calc(
+            case_7.__file__, 'job.ini', mean_hazard_curves='false',
+            poes='0.1')
 
     @attr('qa', 'hazard', 'classical')
     def test_case_8(self):
@@ -181,7 +178,8 @@ class ClassicalTestCase(CalculatorTestCase):
         self.run_calc(
             case_13.__file__, 'job.ini', exports='csv', poes='0.2',
             hazard_calculation_id=str(self.calc.datastore.calc_id),
-            concurrent_tasks='0')
+            concurrent_tasks='0', gsim_logic_tree_file='',
+            source_model_logic_tree_file='')
         [fname] = export(('hmaps', 'csv'), self.calc.datastore)
         self.assertEqualFiles('expected/hazard_map-mean2.csv', fname,
                               delta=1E-5)
@@ -203,12 +201,17 @@ class ClassicalTestCase(CalculatorTestCase):
                                  'hmaps/poe-0.2/rlz-002',
                                  'hmaps/poe-0.2/rlz-003'])
 
-        # test extract/qgis-hazard/rlz-0 also works
-        haz = dict(extract(self.calc.datastore, 'qgis-hazard/rlz-0'))
+        # test extract/hcurves/rlz-0 also works, used by the npz exports
+        haz = dict(extract(self.calc.datastore, 'hcurves'))
+        self.assertEqual(sorted(haz), ['all', 'investigation_time'])
         self.assertEqual(
-            sorted(haz),
-            ['checksum32', 'hcurves-rlz-0', 'hmaps-rlz-0', 'oqparam',
-             'realizations', 'sitecol'])
+            haz['all'].dtype.names, ('lon', 'lat', 'depth', 'mean'))
+        array = haz['all']['mean']
+        self.assertEqual(array.dtype.names, ('PGA', 'SA(0.2)'))
+        self.assertEqual(array['PGA'].dtype.names,
+                         ('0.005', '0.007', '0.0098', '0.0137', '0.0192',
+                          '0.0269', '0.0376', '0.0527', '0.0738', '0.103',
+                          '0.145', '0.203', '0.284'))
 
     @attr('qa', 'hazard', 'classical')
     def test_case_14(self):
@@ -235,22 +238,6 @@ hazard_uhs-mean.csv
         self.assertEqualFiles('expected/hazard_uhs-mean-0.1.xml', fnames[1])
         self.assertEqualFiles('expected/hazard_uhs-mean-0.2.xml', fnames[2])
 
-        # test hmaps geojson export
-        fnames = [f for f in export(('hmaps', 'geojson'), self.calc.datastore)
-                  if 'mean' in f]
-        self.assertEqualFiles(
-            'expected/hazard_map-mean-0.01-PGA.geojson', fnames[0])
-        self.assertEqualFiles(
-            'expected/hazard_map-mean-0.01-SA(0.1).geojson', fnames[1])
-        self.assertEqualFiles(
-            'expected/hazard_map-mean-0.1-PGA.geojson', fnames[2])
-        self.assertEqualFiles(
-            'expected/hazard_map-mean-0.1-SA(0.1).geojson', fnames[3])
-        self.assertEqualFiles(
-            'expected/hazard_map-mean-0.2-PGA.geojson', fnames[4])
-        self.assertEqualFiles(
-            'expected/hazard_map-mean-0.2-SA(0.1).geojson', fnames[5])
-
         # npz exports
         export(('hmaps', 'npz'), self.calc.datastore)
         export(('uhs', 'npz'), self.calc.datastore)
@@ -334,6 +321,12 @@ hazard_uhs-mean.csv
         [fname] = export(('uhs/rlz-1', 'csv'),  self.calc.datastore)
         self.assertEqualFiles('expected/uhs-rlz-1.csv', fname)
 
+        # extracting hmaps
+        hmaps = dict(extract(self.calc.datastore, 'hmaps'))['all']['mean']
+        self.assertEqual(
+            hmaps.dtype.names,
+            ('PGA-0.002105', 'SA(0.2)-0.002105', 'SA(1.0)-0.002105'))
+
     @attr('qa', 'hazard', 'classical')
     def test_case_19(self):
         self.assert_curves_ok([
