@@ -32,7 +32,7 @@ from openquake.baselib import parallel
 from openquake.hazardlib import nrml
 from openquake.risklib import riskinput
 from openquake.commonlib import readinput, source, calc, util
-from openquake.calculators import base, event_based
+from openquake.calculators import base, event_based, getters
 from openquake.calculators.event_based_risk import (
     EbriskCalculator, event_based_risk)
 
@@ -43,7 +43,8 @@ from openquake.hazardlib.geo.geodetic import min_idx_dst, min_geodetic_distance
 from openquake.hazardlib.geo.surface.planar import PlanarSurface
 from openquake.hazardlib.geo.nodalplane import NodalPlane
 from openquake.hazardlib.tom import PoissonTOM
-from openquake.hazardlib.source.rupture import ParametricProbabilisticRupture
+from openquake.hazardlib.source.rupture import (
+    ParametricProbabilisticRupture, EBRupture)
 from openquake.hazardlib.source.characteristic import CharacteristicFaultSource
 from openquake.hazardlib.source.point import PointSource
 from openquake.hazardlib.scalerel.wc1994 import WC1994
@@ -404,6 +405,13 @@ class UCERFSource(object):
         self.msr = msr
         self.mesh_spacing = mesh_spacing
         self.tectonic_region_type = trt
+        self.num_ruptures = 0  # not set yet
+
+    def count_ruptures(self):
+        """
+        The length of the rupture array if the branch_id is set, else 0
+        """
+        return len(self.mags) if hasattr(self, 'mags') else 0
 
     def new(self, grp_id, branch_id):
         """
@@ -641,6 +649,10 @@ class UCERFSource(object):
                 sources.append(ps)
         return sources
 
+    def filter_sites_by_distance_to_source(self, integration_distance, sites):
+        # do not filter
+        return sites
+
 
 def build_idx_set(branch_id, start_date):
     """
@@ -701,20 +713,21 @@ def compute_ruptures(sources, src_filter, gsims, param, monitor):
                         r_sites, rrup = idist.get_closest(sitecol, rup)
                     except FarAwayRupture:
                         continue
-                    indices = r_sites.indices
+                    indices = (numpy.arange(len(r_sites)) if r_sites.indices
+                               is None else r_sites.indices)
                     events = []
                     for _ in range(n_occ):
-                        events.append((0, ses_idx, sample))
+                        events.append((0, src.src_group_id, ses_idx, sample))
                     if events:
                         evs = numpy.array(events, calc.event_dt)
                         ebruptures.append(
-                            calc.EBRupture(rup, indices, evs,
-                                           src.src_group_id, serial))
+                            EBRupture(rup, indices, evs, serial))
                         serial += 1
     res.num_events = event_based.set_eids(ebruptures)
     res[src.src_group_id] = ebruptures
-    res.calc_times[src.src_group_id] = (
-        src.source_id, len(sitecol), time.time() - t0)
+    res.calc_times[src.src_group_id] = {
+        src.source_id:
+        numpy.array([src.weight, len(sitecol), time.time() - t0, 1])}
     if not param['save_ruptures']:
         res.events_by_grp = {grp_id: event_based.get_events(res[grp_id])
                              for grp_id in res}
@@ -727,7 +740,7 @@ def get_composite_source_model(oq):
     :param oq: :class:`openquake.commonlib.oqvalidation.OqParam` instance
     :returns: a `class:`openquake.commonlib.source.CompositeSourceModel`
     """
-    [src_group] = nrml.parse(
+    [src_group] = nrml.to_python(
         oq.inputs["source_model"],
         SourceConverter(oq.investigation_time, oq.rupture_mesh_spacing))
     source_models = []
@@ -737,7 +750,7 @@ def get_composite_source_model(oq):
         sg = copy.copy(src_group)
         sg.id = sm.ordinal
         sm.src_groups = [sg]
-        sg.sources = [sg[0].new(sm.ordinal, sm.name)]
+        sg.sources = [sg[0].new(sm.ordinal, sm.names)]
         source_models.append(sm)
     return source.CompositeSourceModel(gsim_lt, smlt, source_models)
 
@@ -768,6 +781,7 @@ class UCERFRuptureCalculator(event_based.EventBasedRuptureCalculator):
         if not self.oqparam.imtls:
             raise ValueError('Missing intensity_measure_types!')
         self.rupser = calc.RuptureSerializer(self.datastore)
+        self.precomputed_gmfs = False
 
     def gen_args(self, csm, monitor):
         """
@@ -824,16 +838,18 @@ def compute_losses(ssm, src_filter, param, riskmodel,
     samples = ssm.info.get_samples_by_grp()
     num_rlzs = len(rlzs_assoc.realizations)
     rlzs_by_gsim = rlzs_assoc.get_rlzs_by_gsim(DEFAULT_TRT)
-    getter = riskinput.GmfGetter(
+    getter = getters.GmfGetter(
         rlzs_by_gsim, ebruptures, src_filter.sitecol, imts, min_iml,
-        trunc_level, correl_model, samples[grp_id])
-    ri = riskinput.RiskInputFromRuptures(getter)
+        src_filter.integration_distance, trunc_level, correl_model,
+        samples[grp_id])
+    ri = riskinput.RiskInput(getter, param['assetcol'].assets_by_site())
     res.append(event_based_risk(ri, riskmodel, param, monitor))
     res.sm_id = ssm.sm_id
     res.num_events = len(ri.hazard_getter.eids)
     start = res.sm_id * num_rlzs
     res.rlz_slice = slice(start, start + num_rlzs)
     res.events_by_grp = ruptures_by_grp.events_by_grp
+    res.eff_ruptures = ruptures_by_grp.eff_ruptures
     return res
 
 
@@ -867,10 +883,12 @@ class UCERFRiskCalculator(EbriskCalculator):
                               ('loss', (F32, (self.L, self.I)))])
         monitor = self.monitor('compute_losses')
         for sm in self.csm.source_models:
+            if sm.samples > 1:
+                logging.warn('Sampling in ucerf_risk is untested')
             ssm = self.csm.get_model(sm.ordinal)
             for ses_idx in range(1, oq.ses_per_logic_tree_path + 1):
                 param = dict(ses_seeds=[(ses_idx, oq.ses_seed + ses_idx)],
-                             samples=1, assetcol=self.assetcol,
+                             samples=sm.samples, assetcol=self.assetcol,
                              save_ruptures=False,
                              ses_ratio=oq.ses_ratio,
                              avg_losses=oq.avg_losses,
@@ -882,9 +900,13 @@ class UCERFRiskCalculator(EbriskCalculator):
                        correl_model, min_iml, monitor)
 
     def execute(self):
+        self.riskmodel.taxonomy = self.assetcol.tagcol.taxonomy
         num_rlzs = len(self.rlzs_assoc.realizations)
-        self.grp_trt = self.csm_info.grp_trt()
+        self.grp_trt = self.csm_info.grp_by("trt")
         res = parallel.Starmap(compute_losses, self.gen_args()).submit_all()
         self.vals = self.assetcol.values()
+        self.eff_ruptures = AccumDict(accum=0)
         num_events = self.save_results(res, num_rlzs)
+        self.csm.info.update_eff_ruptures(self.eff_ruptures)
+        self.datastore['csm_info'] = self.csm.info
         return num_events
