@@ -23,12 +23,10 @@ import h5py
 
 from openquake.baselib import hdf5
 from openquake.baselib.python3compat import decode
-from openquake.hazardlib.geo.mesh import (
-    surface_to_mesh, point3d, RectangularMesh)
-from openquake.hazardlib.source.rupture import BaseRupture, EBRupture
+from openquake.hazardlib.geo.mesh import surface_to_mesh, point3d
 from openquake.hazardlib.gsim.base import ContextMaker
 from openquake.hazardlib.imt import from_string
-from openquake.hazardlib import geo, calc, probability_map, stats
+from openquake.hazardlib import calc, probability_map
 
 TWO16 = 2 ** 16
 MAX_INT = 2 ** 31 - 1  # this is used in the random number generator
@@ -43,11 +41,10 @@ F32 = numpy.float32
 U64 = numpy.uint64
 F64 = numpy.float64
 
-event_dt = numpy.dtype([('eid', U64), ('ses', U32), ('sample', U32)])
-
-sids_dt = h5py.special_dtype(vlen=U32)
-
-BaseRupture.init()  # initialize rupture codes
+EVENTS = -2
+NBYTES = -1
+event_dt = numpy.dtype([('eid', U64), ('grp_id', U16), ('ses', U32),
+                        ('sample', U32)])
 
 # ############## utilities for the classical calculator ############### #
 
@@ -77,123 +74,6 @@ def convert_to_array(pmap, nsites, imtls):
                 idx += 1
     return curves
 
-
-class PmapGetter(object):
-    """
-    Read hazard curves from the datastore for all realizations or for a
-    specific realization.
-
-    :param dstore: a DataStore instance
-    :param sids: the subset of sites to consider (if None, all sites)
-    :param rlzs_assoc: a RlzsAssoc instance (if None, infers it)
-    """
-    def __init__(self, dstore, sids=None, rlzs_assoc=None):
-        self.rlzs_assoc = rlzs_assoc or dstore['csm_info'].get_rlzs_assoc()
-        self.dstore = dstore
-        self.weights = [rlz.weight for rlz in self.rlzs_assoc.realizations]
-        self.num_levels = len(self.dstore['oqparam'].imtls.array)
-        self.sids = sids
-        self.nbytes = 0
-        if sids is None:
-            self.sids = dstore['sitecol'].complete.sids
-        # populate _pmap_by_grp
-        self._pmap_by_grp = {}
-        if 'poes' in self.dstore:
-            # build probability maps restricted to the given sids
-            for grp, dset in self.dstore['poes'].items():
-                sid2idx = {sid: i for i, sid in enumerate(dset.attrs['sids'])}
-                L, I = dset.shape[1:]
-                pmap = probability_map.ProbabilityMap(L, I)
-                for sid in self.sids:
-                    try:
-                        idx = sid2idx[sid]
-                    except KeyError:
-                        continue
-                    else:
-                        pmap[sid] = probability_map.ProbabilityCurve(dset[idx])
-                self._pmap_by_grp[grp] = pmap
-                self.nbytes += pmap.nbytes
-
-    def get(self, rlzi, grp=None):
-        """
-        :param rlzi: a realization index
-        :param grp: None (all groups) or a string of the form "grp-XX"
-        :returns: the hazard curves for the given realization
-        """
-        assert self.sids is not None
-        pmap = probability_map.ProbabilityMap(self.num_levels, 1)
-        grps = [grp] if grp is not None else sorted(self._pmap_by_grp)
-        array = self.rlzs_assoc.by_grp()
-        for grp in grps:
-            for gsim_idx, rlzis in array[grp]:
-                for r in rlzis:
-                    if r == rlzi:
-                        pmap |= self._pmap_by_grp[grp].extract(gsim_idx)
-                        break
-        return pmap
-
-    def get_pmaps(self, sids):  # used in classical
-        """
-        :param sids: an array of S site IDs
-        :returns: a list of R probability maps
-        """
-        return self.rlzs_assoc.combine_pmaps(self._pmap_by_grp)
-
-    def get_hcurves(self, imtls):
-        """
-        :param imtls: intensity measure types and levels
-        :returns: an array of (R, N) hazard curves
-        """
-        assert self.sids is not None, 'PmapGetter not bound to sids'
-        pmaps = [pmap.convert2(imtls, self.sids)
-                 for pmap in self.get_pmaps(self.sids)]
-        return numpy.array(pmaps)
-
-    def items(self, kind=''):
-        """
-        Extract probability maps from the datastore, possibly generating
-        on the fly the ones corresponding to the individual realizations.
-        Yields pairs (tag, pmap).
-
-        :param kind:
-            the kind of PoEs to extract; if not given, returns the realization
-            if there is only one or the statistics otherwise.
-        """
-        num_rlzs = len(self.weights)
-        if not kind:  # use default
-            if 'hcurves' in self.dstore:
-                for k in sorted(self.dstore['hcurves']):
-                    yield k, self.dstore['hcurves/' + k]
-            elif num_rlzs == 1:
-                yield 'rlz-000', self.get(0)
-            return
-        if 'poes' in self.dstore and kind in ('rlzs', 'all'):
-            for rlzi in range(num_rlzs):
-                hcurves = self.get(rlzi)
-                yield 'rlz-%03d' % rlzi, hcurves
-        elif 'poes' in self.dstore and kind.startswith('rlz-'):
-            yield kind, self.get(int(kind[4:]))
-        if 'hcurves' in self.dstore and kind in ('stats', 'all'):
-            for k in sorted(self.dstore['hcurves']):
-                yield k, self.dstore['hcurves/' + k]
-
-    def get_mean(self, grp=None):
-        """
-        Compute the mean curve as a ProbabilityMap
-
-        :param grp:
-            if not None must be a string of the form "grp-XX"; in that case
-            returns the mean considering only the contribution for group XX
-        """
-        if self.sids is None:
-            self.sids = self.dstore['sitecol'].complete.sids
-        if len(self.weights) == 1:  # one realization
-            return self.get(0, grp)
-        else:  # multiple realizations, assume hcurves/mean is there
-            dic = ({g: self.dstore['poes/' + g] for g in self.dstore['poes']}
-                   if grp is None else {grp: self.dstore['poes/' + grp]})
-            return self.rlzs_assoc.compute_pmap_stats(dic, [stats.mean_curve])
-
 # ######################### hazard maps ################################### #
 
 # cutoff value for the poe
@@ -440,7 +320,7 @@ class RuptureData(object):
                              set('mag strike dip rake hypo_depth'.split()))
         self.dt = numpy.dtype([
             ('rup_id', U32), ('multiplicity', U16), ('eidx', U32),
-            ('numsites', U32), ('occurrence_rate', F64),
+            ('occurrence_rate', F64),
             ('mag', F32), ('lon', F32), ('lat', F32), ('depth', F32),
             ('strike', F32), ('dip', F32), ('rake', F32),
             ('boundary', hdf5.vstr)] + [(param, F32) for param in self.params])
@@ -464,7 +344,7 @@ class RuptureData(object):
             except AttributeError:  # for nonparametric sources
                 rate = numpy.nan
             data.append(
-                (ebr.serial, ebr.multiplicity, ebr.eidx1, len(ebr.sids), rate,
+                (ebr.serial, ebr.multiplicity, ebr.eidx1, rate,
                  rup.mag, point.x, point.y, point.z, rup.surface.get_strike(),
                  rup.surface.get_dip(), rup.rake,
                  'MULTIPOLYGON(%s)' % decode(bounds)) + ruptparams)
@@ -477,7 +357,7 @@ class RuptureSerializer(object):
     `ruptures` and `sids`.
     """
     rupture_dt = numpy.dtype([
-        ('serial', U32), ('code', U8), ('sidx', U32),
+        ('serial', U32), ('grp_id', U16), ('code', U8),
         ('eidx1', U32), ('eidx2', U32), ('pmfx', I32), ('seed', U32),
         ('mag', F32), ('rake', F32), ('occurrence_rate', F32),
         ('hypo', point3d), ('sx', U16), ('sy', U8), ('sz', U16),
@@ -506,7 +386,7 @@ class RuptureSerializer(object):
             assert sz < TWO16, 'The rupture mesh spacing is too small'
             hypo = rup.hypocenter.x, rup.hypocenter.y, rup.hypocenter.z
             rate = getattr(rup, 'occurrence_rate', numpy.nan)
-            tup = (ebrupture.serial, rup.code, ebrupture.sidx,
+            tup = (ebrupture.serial, ebrupture.grp_id, rup.code,
                    ebrupture.eidx1, ebrupture.eidx2,
                    getattr(ebrupture, 'pmfx', -1),
                    rup.seed, rup.mag, rup.rake, rate, hypo,
@@ -517,8 +397,6 @@ class RuptureSerializer(object):
 
     def __init__(self, datastore):
         self.datastore = datastore
-        self.sids = {}  # dictionary sids -> sidx
-        self.data = []
         self.nbytes = 0
 
     def save(self, ebruptures, eidx=0):
@@ -529,30 +407,21 @@ class RuptureSerializer(object):
         :param eidx: the last event index saved
         """
         pmfbytes = 0
-        # set the reference to the sids (sidx) correctly
         for ebr in ebruptures:
             mul = ebr.multiplicity
             ebr.eidx1 = eidx
             ebr.eidx2 = eidx + mul
             eidx += mul
-            sids_tup = tuple(ebr.sids)
-            try:
-                ebr.sidx = self.sids[sids_tup]
-            except KeyError:
-                ebr.sidx = self.sids[sids_tup] = len(self.sids)
-                self.data.append(ebr.sids)
-
             rup = ebr.rupture
             if hasattr(rup, 'pmf'):
                 pmfs = numpy.array([(ebr.serial, rup.pmf)], self.pmfs_dt)
-                dset = self.datastore.extend(
-                    'pmfs/grp-%02d' % ebr.grp_id, pmfs)
+                dset = self.datastore.extend('pmfs', pmfs)
                 ebr.pmfx = len(dset) - 1
                 pmfbytes += self.pmfs_dt.itemsize + rup.pmf.nbytes
 
         # store the ruptures in a compact format
         array, nbytes = self.get_array_nbytes(ebruptures)
-        key = 'ruptures/grp-%02d' % ebr.grp_id
+        key = 'ruptures'
         try:
             dset = self.datastore.getitem(key)
         except KeyError:  # not created yet
@@ -570,72 +439,4 @@ class RuptureSerializer(object):
         self.datastore.flush()
 
     def close(self):
-        """
-        Flush the ruptures and the site IDs on the datastore
-        """
-        self.sids.clear()
-        if self.data:
-            self.datastore.save_vlen('sids', self.data)
-            del self.data[:]
-
-
-def get_ruptures(dstore, events, grp_id):
-    """
-    Extracts the ruptures of the given grp_id
-    """
-    return _get_ruptures(dstore, events, [grp_id], None)
-
-
-def _get_ruptures(dstore, events, grp_ids, rup_id):
-    oq = dstore['oqparam']
-    grp_trt = dstore['csm_info'].grp_trt()
-    for grp_id in grp_ids:
-        trt = grp_trt[grp_id]
-        grp = 'grp-%02d' % grp_id
-        try:
-            recs = dstore['ruptures/' + grp]
-        except KeyError:  # no ruptures in grp
-            continue
-        for rec in recs:
-            if rup_id is not None and rup_id != rec['serial']:
-                continue
-            mesh = rec['points'].reshape(rec['sx'], rec['sy'], rec['sz'])
-            rupture_cls, surface_cls, source_cls = BaseRupture.types[
-                rec['code']]
-            rupture = object.__new__(rupture_cls)
-            rupture.surface = object.__new__(surface_cls)
-            # MISSING: case complex_fault_mesh_spacing != rupture_mesh_spacing
-            if 'Complex' in surface_cls.__name__:
-                mesh_spacing = oq.complex_fault_mesh_spacing
-            else:
-                mesh_spacing = oq.rupture_mesh_spacing
-            rupture.source_typology = source_cls
-            rupture.mag = rec['mag']
-            rupture.rake = rec['rake']
-            rupture.seed = rec['seed']
-            rupture.hypocenter = geo.Point(*rec['hypo'])
-            rupture.occurrence_rate = rec['occurrence_rate']
-            rupture.tectonic_region_type = trt
-            pmfx = rec['pmfx']
-            if pmfx != -1:
-                rupture.pmf = dstore['pmfs/' + grp][pmfx]
-            if surface_cls is geo.PlanarSurface:
-                rupture.surface = geo.PlanarSurface.from_array(
-                    mesh_spacing, rec['points'])
-            elif surface_cls.__name__.endswith('MultiSurface'):
-                rupture.surface.__init__([
-                    geo.PlanarSurface.from_array(mesh_spacing, m1.flatten())
-                    for m1 in mesh])
-            else:  # fault surface, strike and dip will be computed
-                rupture.surface.strike = rupture.surface.dip = None
-                m = mesh[0]
-                rupture.surface.mesh = RectangularMesh(
-                    m['lon'], m['lat'], m['depth'])
-            sids = dstore['sids'][rec['sidx']]
-            evs = events[rec['eidx1']:rec['eidx2']]
-            ebr = EBRupture(rupture, sids, evs, grp_id, rec['serial'])
-            ebr.eidx1 = rec['eidx1']
-            ebr.eidx2 = rec['eidx2']
-            ebr.sidx = rec['sidx']
-            # not implemented: rupture_slip_direction
-            yield ebr
+        pass
