@@ -21,7 +21,7 @@ import logging
 import functools
 import numpy
 
-from openquake.baselib import parallel
+from openquake.baselib import parallel, datastore
 from openquake.baselib.general import DictArray
 from openquake.hazardlib.imt import from_string
 from openquake.hazardlib import correlation, stats
@@ -40,8 +40,7 @@ class OqParam(valid.ParamSet):
         vs30='reference_vs30_value',
         z1pt0='reference_depth_to_1pt0km_per_sec',
         z2pt5='reference_depth_to_2pt5km_per_sec',
-        backarc='reference_backarc',
-    )
+        backarc='reference_backarc')
     asset_loss_table = valid.Param(valid.boolean, False)
     area_source_discretization = valid.Param(
         valid.NoneOr(valid.positivefloat), None)
@@ -57,6 +56,7 @@ class OqParam(valid.ParamSet):
     conditional_loss_poes = valid.Param(valid.probabilities, [])
     continuous_fragility_discretization = valid.Param(valid.positiveint, 20)
     description = valid.Param(valid.utf8_not_empty)
+    disagg_by_src = valid.Param(valid.boolean, False)
     disagg_outputs = valid.Param(valid.disagg_outputs, None)
     distance_bin_width = valid.Param(valid.positivefloat)
     mag_bin_width = valid.Param(valid.positivefloat)
@@ -179,8 +179,14 @@ class OqParam(valid.ParamSet):
             delattr(self, 'intensity_measure_types')
         self._file_type, self._risk_files = get_risk_files(self.inputs)
 
+        self.check_source_model()
+        if self.hazard_precomputed():
+            self.check_missing('site_model', 'warn')
+            self.check_missing('gsim_logic_tree', 'warn')
+            self.check_missing('source_model_logic_tree', 'warn')
+
         # check the gsim_logic_tree
-        if 'gsim_logic_tree' in self.inputs:
+        if self.inputs.get('gsim_logic_tree'):
             if self.gsim:
                 raise InvalidFile('%s: if `gsim_logic_tree_file` is set, there'
                                   ' must be no `gsim` key' % job_ini)
@@ -202,7 +208,16 @@ class OqParam(valid.ParamSet):
         elif self.gsim is not None:
             self.check_gsims([self.gsim])
 
-        self.check_source_model()
+        # checks for hazard outputs
+        if not self.hazard_stats():
+            if self.uniform_hazard_spectra:
+                raise InvalidFile(
+                    '%(job_ini)s: uniform_hazard_spectra=true is inconsistent '
+                    'with mean_hazard_curves=false' % self.inputs)
+            elif self.hazard_maps:
+                raise InvalidFile(
+                    '%(job_ini)s: hazard_maps=true is inconsistent '
+                    'with mean_hazard_curves=false' % self.inputs)
 
         # checks for disaggregation
         if self.calculation_mode == 'disaggregation':
@@ -233,8 +248,8 @@ class OqParam(valid.ParamSet):
         elif (self.calculation_mode == 'event_based_risk'
               and self.conditional_loss_poes and not self.asset_loss_table):
             raise InvalidFile(
-                '%s: conditional_loss_poes is set, but the loss maps cannot '
-                'be generated unless you set asset_loss_table=true' % job_ini)
+                '%s: asset_loss_table is not set, probably you want to remove'
+                ' conditional_loss_poes' % job_ini)
 
         # check for GMFs from file
         if (self.inputs.get('gmfs', '').endswith('.csv') and not self.sites and
@@ -503,7 +518,8 @@ class OqParam(valid.ParamSet):
         """
         Invalid maximum_distance={maximum_distance}: {error}
         """
-        if 'source_model_logic_tree' not in self.inputs:
+        if (not self.inputs.get('source_model_logic_tree') or not
+                self.inputs.get('gsim_logic_tree')):
             return True  # don't apply validation
         gsim_lt = self.inputs['gsim_logic_tree']
         trts = set(self.maximum_distance)
@@ -636,3 +652,26 @@ class OqParam(valid.ParamSet):
             raise ValueError('Missing source_model_logic_tree in %s '
                              'or missing --hc option' %
                              self.inputs.get('job_ini', 'job_ini'))
+
+    def check_missing(self, param, action):
+        """
+        Make sure the given parameter is missing in the job.ini file
+        """
+        assert action in ('warn', 'error'), action
+        if self.inputs.get(param):
+            msg = 'Please remove %s_file from %s, it makes no sense in %s' % (
+                param, self.inputs['job_ini'], self.calculation_mode)
+            if action == 'error':
+                raise InvalidFile(msg)
+            else:
+                logging.warn(msg)
+
+    def hazard_precomputed(self):
+        """
+        :returns: True if the hazard is precomputed
+        """
+        if 'gmfs' in self.inputs or 'hazard_curves' in self.inputs:
+            return True
+        elif self.hazard_calculation_id:
+            parent = list(datastore.read(self.hazard_calculation_id))
+            return 'gmf_data' in parent or 'poes' in parent
