@@ -31,12 +31,12 @@ from openquake.baselib.python3compat import decode
 from openquake.baselib.general import (
     groupby, group_array, block_splitter, writetmp, AccumDict)
 from openquake.hazardlib import (
-    nrml, sourceconverter, InvalidFile, probability_map, stats)
+    nrml, source, sourceconverter, InvalidFile, probability_map, stats)
+from openquake.hazardlib.gsim.gsim_table import GMPETable
 from openquake.commonlib import logictree
 
 
-MINWEIGHT = sourceconverter.MINWEIGHT
-MAXWEIGHT = 4E6  # heuristic, set by M. Simionato
+MINWEIGHT = source.MINWEIGHT
 MAX_INT = 2 ** 31 - 1
 TWO16 = 2 ** 16
 U16 = numpy.uint16
@@ -44,6 +44,15 @@ U32 = numpy.uint32
 I32 = numpy.int32
 F32 = numpy.float32
 weight = operator.attrgetter('weight')
+rlz_dt = numpy.dtype([('uid', 'S200'), ('model', 'S200'),
+                      ('gsims', 'S100'), ('weight', F32)])
+
+
+def gsim_names(rlz):
+    """
+    Names of the underlying GSIMs separated by spaces
+    """
+    return ' '.join(str(v) for v in rlz.gsim_rlz.value)
 
 
 class LtRealization(object):
@@ -290,6 +299,7 @@ src_group_dt = numpy.dtype(
     [('grp_id', U32),
      ('trti', U16),
      ('effrup', I32),
+     ('totrup', I32),
      ('sm_id', U32)])
 
 
@@ -316,6 +326,17 @@ def accept_path(path, ref_path):
     return True
 
 
+def get_totrup(data):
+    """
+    :param data: a record with a field `totrup`, possibily missing
+    """
+    try:
+        totrup = data['totrup']
+    except ValueError:  # engine older than 2.9
+        totrup = 0
+    return totrup
+
+
 class CompositionInfo(object):
     """
     An object to collect information about the composition of
@@ -349,7 +370,7 @@ class CompositionInfo(object):
         self.init()
 
     def init(self):
-        self.trt_by_grp = self.grp_trt()
+        self.trt_by_grp = self.grp_by("trt")
         if self.num_samples:
             self.seed_samples_by_grp = {}
             seed = self.seed
@@ -404,24 +425,31 @@ class CompositionInfo(object):
         # with this CompositionInfo instances will be unpickled correctly
         return self.seed, self.num_samples, self.source_models
 
-    def __toh5__(self):
+    def trt2i(self):
+        """
+        :returns: trt -> trti
+        """
         trts = sorted(set(src_group.trt for sm in self.source_models
                           for src_group in sm.src_groups))
-        trti = {trt: i for i, trt in enumerate(trts)}
+        return {trt: i for i, trt in enumerate(trts)}
+
+    def __toh5__(self):
         data = []
+        trti = self.trt2i()
         for sm in self.source_models:
             for src_group in sm.src_groups:
                 # the number of effective realizations is set by get_rlzs_assoc
                 data.append((src_group.id, trti[src_group.trt],
-                             src_group.eff_ruptures, sm.ordinal))
-        lst = [(sm.name, sm.weight, '_'.join(sm.path),
+                             src_group.eff_ruptures, src_group.tot_ruptures,
+                             sm.ordinal))
+        lst = [(sm.names, sm.weight, '_'.join(sm.path),
                 sm.num_gsim_paths, sm.samples)
                for i, sm in enumerate(self.source_models)]
         return (dict(
             sg_data=numpy.array(data, src_group_dt),
             sm_data=numpy.array(lst, source_model_dt)),
                 dict(seed=self.seed, num_samples=self.num_samples,
-                     trts=hdf5.array_of_vstr(trts),
+                     trts=hdf5.array_of_vstr(sorted(trti)),
                      gsim_lt_xml=str(self.gsim_lt),
                      gsim_fname=self.gsim_lt.fname,
                      tot_weight=self.tot_weight))
@@ -433,20 +461,11 @@ class CompositionInfo(object):
         vars(self).update(attrs)
         self.gsim_fname = decode(self.gsim_fname)
         if self.gsim_fname.endswith('.xml'):
+            # otherwise it would look in the current directory
+            GMPETable.GMPE_DIR = os.path.dirname(self.gsim_fname)
             trts = sorted(self.trts)
-            if 'gmpe_table' in self.gsim_lt_xml:
-                # the canadian gsims depends on external files which are not
-                # in the datastore; I am storing the path to the original
-                # file so that the external files can be found; unfortunately,
-                # this means that copying the datastore on a different machine
-                # and exporting from there works only if the gsim_fname and all
-                # the external files are copied in the exact same place
-                self.gsim_lt = logictree.GsimLogicTree(self.gsim_fname, trts)
-            else:
-                # regular case: read the logic tree from self.gsim_lt_xml,
-                # so that you do not need to copy anything except the datastore
-                tmp = writetmp(self.gsim_lt_xml, suffix='.xml')
-                self.gsim_lt = logictree.GsimLogicTree(tmp, trts)
+            tmp = writetmp(self.gsim_lt_xml, suffix='.xml')
+            self.gsim_lt = logictree.GsimLogicTree(tmp, trts)
         else:  # fake file with the name of the GSIM
             self.gsim_lt = logictree.GsimLogicTree.from_(self.gsim_fname)
         self.source_models = []
@@ -454,8 +473,9 @@ class CompositionInfo(object):
             tdata = sg_data[sm_id]
             srcgroups = [
                 sourceconverter.SourceGroup(
-                    self.trts[trti], id=grp_id, eff_ruptures=effrup)
-                for grp_id, trti, effrup, sm_id in tdata if effrup]
+                    self.trts[data['trti']], id=data['grp_id'],
+                    eff_ruptures=data['effrup'], tot_ruptures=get_totrup(data))
+                for data in tdata if data['effrup']]
             path = tuple(str(decode(rec['path'])).split('_'))
             trts = set(sg.trt for sg in srcgroups)
             num_gsim_paths = self.gsim_lt.reduce(trts).get_num_paths()
@@ -481,12 +501,31 @@ class CompositionInfo(object):
         trts = set(sg.trt for sg in source_model.src_groups)
         return self.gsim_lt.reduce(trts).get_num_paths()
 
-    def get_rlzs_assoc(self, count_ruptures=None,
-                       sm_lt_path=None, trts=None):
+    @property
+    def rlzs(self):
+        """
+        :returns: an array of realizations
+        """
+        realizations = self.get_rlzs_assoc().realizations
+        sm_by_rlz = self.get_sm_by_rlz(
+            realizations) or collections.defaultdict(lambda: 'NA')
+        return numpy.array(
+            [(r.uid, sm_by_rlz[r], gsim_names(r), r.weight)
+             for r in realizations], rlz_dt)
+
+    def update_eff_ruptures(self, count_ruptures):
+        """
+        :param count_ruptures: function or dict src_group_id -> num_ruptures
+        """
+        for smodel in self.source_models:
+            for sg in smodel.src_groups:
+                sg.eff_ruptures = (count_ruptures(sg.id)
+                                   if callable(count_ruptures)
+                                   else count_ruptures[sg.id])
+
+    def get_rlzs_assoc(self, sm_lt_path=None, trts=None):
         """
-        :param count_ruptures: function src_group_id -> num_ruptures
         :param sm_lt_path: logic tree path tuple used to select a source model
-        :param gsim_lt_path: gsim logic tree path tuple
         :param trts: tectonic region types to accept
         """
         assoc = RlzsAssoc(self)
@@ -500,8 +539,6 @@ class CompositionInfo(object):
             # collect the effective tectonic region types and ruptures
             trts_ = set()
             for sg in smodel.src_groups:
-                if count_ruptures:
-                    sg.eff_ruptures = count_ruptures(sg.id)
                 if sg.eff_ruptures:
                     if (trts and sg.trt in trts) or not trts:
                         trts_.add(sg.trt)
@@ -511,7 +548,7 @@ class CompositionInfo(object):
                 before = self.gsim_lt.get_num_paths()
                 gsim_lt = self.gsim_lt.reduce(trts_)
                 after = gsim_lt.get_num_paths()
-                if count_ruptures and before > after:
+                if before > after:
                     logging.warn('Reducing the logic tree of %s from %d to %d '
                                  'realizations', smodel.name, before, after)
                 gsim_rlzs = list(gsim_lt)
@@ -551,7 +588,7 @@ class CompositionInfo(object):
         for sm in self.source_models:
             for rlz in realizations:
                 if rlz.sm_lt_path == sm.path:
-                    dic[rlz] = sm.name
+                    dic[rlz] = sm.names
         return dic
 
     def get_sm_by_grp(self):
@@ -561,14 +598,14 @@ class CompositionInfo(object):
         return {grp.id: sm.ordinal for sm in self.source_models
                 for grp in sm.src_groups}
 
-    def grp_trt(self):
+    def grp_by(self, name):
         """
         :returns: a dictionary grp_id -> TRT string
         """
         dic = {}
         for smodel in self.source_models:
             for src_group in smodel.src_groups:
-                dic[src_group.id] = src_group.trt
+                dic[src_group.id] = getattr(src_group, name)
         return dic
 
     def _get_rlzs(self, smodel, all_rlzs, seed):
@@ -584,7 +621,7 @@ class CompositionInfo(object):
         if len(rlzs) > TWO16:
             raise ValueError(
                 'The source model %s has %d realizations, the maximum '
-                'is %d' % (smodel.name, len(rlzs), TWO16))
+                'is %d' % (smodel.names, len(rlzs), TWO16))
         return rlzs
 
     def __repr__(self):
@@ -592,7 +629,7 @@ class CompositionInfo(object):
         for sm in self.source_models:
             info_by_model[sm.path] = (
                 '_'.join(map(decode, sm.path)),
-                decode(sm.name),
+                decode(sm.names),
                 [sg.id for sg in sm.src_groups],
                 sm.weight,
                 self.get_num_rlzs(sm))
@@ -633,6 +670,26 @@ class CompositeSourceModel(collections.Sequence):
         else:
             self.has_dupl_sources = len(dupl_sources)
 
+    def grp_by_src(self):
+        """
+        :returns: a new CompositeSourceModel with one group per source
+        """
+        smodels = []
+        grp_id = 0
+        for sm in self.source_models:
+            src_groups = []
+            smodel = sm.__class__(sm.names, sm.weight, sm.path, src_groups,
+                                  sm.num_gsim_paths, sm.ordinal, sm.samples)
+            for sg in sm.src_groups:
+                for src in sg.sources:
+                    src.src_group_id = grp_id
+                    src_groups.append(
+                        sourceconverter.SourceGroup(
+                            sg.trt, [src], name=src.source_id, id=grp_id))
+                    grp_id += 1
+            smodels.append(smodel)
+        return self.__class__(self.gsim_lt, self.source_model_lt, smodels)
+
     def get_model(self, sm_id):
         """
         Extract a CompositeSourceModel instance containing the single
@@ -647,7 +704,7 @@ class CompositeSourceModel(collections.Sequence):
                          for src in sg.sources)
         return new
 
-    def filter(self, src_filter):
+    def filter(self, src_filter):  # called once per tile
         """
         Generate a new CompositeSourceModel by filtering the sources on
         the given site collection.
@@ -655,29 +712,36 @@ class CompositeSourceModel(collections.Sequence):
         :param sitecol: a SiteCollection instance
         :para src_filter: a SourceFilter instance
         """
+        ngsims = {trt: len(gs) for trt, gs in self.gsim_lt.values.items()}
         source_models = []
         weight = 0
         for sm in self.source_models:
             src_groups = []
             for src_group in sm.src_groups:
+                mutex = getattr(src_group, 'src_interdep', None) == 'mutex'
+                self.add_infos(src_group.sources)  # unsplit sources
                 sources = []
                 for src in src_group.sources:
-                    if hasattr(src, '__iter__'):  # MultiPointSource
-                        sources.extend(src)
+                    if hasattr(src, '__iter__') and not mutex:
+                        # MultiPoint, AreaSource, NonParametric
+                        # NB: source.split_source is cached
+                        sources.extend(source.split_source(src))
                     else:
+                        # mutex sources cannot be split
                         sources.append(src)
                 sg = copy.copy(src_group)
                 sg.sources = []
-                for src, sites in src_filter(sources):
+                for src, _sites in src_filter(sources):
                     sg.sources.append(src)
+                    src.ngsims = ngsims[src.tectonic_region_type]
                     weight += src.weight
                 src_groups.append(sg)
             newsm = logictree.SourceModel(
-                sm.name, sm.weight, sm.path, src_groups,
+                sm.names, sm.weight, sm.path, src_groups,
                 sm.num_gsim_paths, sm.ordinal, sm.samples)
             source_models.append(newsm)
         new = self.__class__(self.gsim_lt, self.source_model_lt, source_models)
-        new.weight = weight
+        new.weight = new.info.tot_weight = weight
         new.src_filter = src_filter
         return new
 
@@ -776,7 +840,7 @@ class CompositeSourceModel(collections.Sequence):
         Generate unique seeds for each rupture with numpy.arange.
         This should be called only in event based calculators
         """
-        n = sum(sg.tot_ruptures() for sg in self.src_groups)
+        n = sum(sg.tot_ruptures for sg in self.src_groups)
         rup_serial = numpy.arange(n, dtype=numpy.uint32)
         start = 0
         for sg in self.src_groups:
@@ -785,33 +849,29 @@ class CompositeSourceModel(collections.Sequence):
                 src.serial = rup_serial[start:start + nr]
                 start += nr
 
-    def get_maxweight(self, concurrent_tasks):
+    def get_maxweight(self, concurrent_tasks, minweight=MINWEIGHT):
         """
         Return an appropriate maxweight for use in the block_splitter
         """
         ct = concurrent_tasks or 1
         mw = math.ceil(self.weight / ct)
-        if mw < MINWEIGHT:
-            mw = MINWEIGHT
-        elif mw > MAXWEIGHT:
-            mw = MAXWEIGHT
-        return mw
+        return max(mw, minweight)
 
     def add_infos(self, sources):
         """
         Populate the .infos dictionary (grp_id, src_id) -> <SourceInfo>
         """
         for src in sources:
-            for grp_id in src.src_group_ids:
-                self.infos[grp_id, src.source_id] = SourceInfo(src)
+            self.infos[src.source_id] = SourceInfo(src)
 
-    def split_in_blocks(self, maxweight, sources):
+    def split_in_blocks(self, maxweight, sources, weight=weight):
         """
         Split a set of sources in blocks of weight up to maxweight; heavy
         sources (i.e. with weight > maxweight) are split.
 
         :param maxweight: maximum weight of a block
         :param sources: sources of the same source group
+        :param weight: source weight function
         :yields: blocks of sources of weight around maxweight
         """
         sources.sort(key=weight)
@@ -824,7 +884,7 @@ class CompositeSourceModel(collections.Sequence):
         # yield heavy sources in blocks
         heavy = [src for src in sources if src.weight > maxweight]
         for src in heavy:
-            srcs = [s for s in split_source(src)
+            srcs = [s for s in source.split_source(src)
                     if self.src_filter.get_close_sites(s) is not None]
             for block in block_splitter(srcs, maxweight, weight):
                 yield block
@@ -851,32 +911,6 @@ class CompositeSourceModel(collections.Sequence):
         return len(self.source_models)
 
 
-split_map = {}  # src -> split sources
-
-
-def split_source(src):
-    """
-    :param src: a source to split
-    :returns: a list of split sources
-    """
-    has_serial = hasattr(src, 'serial')
-    start = 0
-    try:
-        splits = split_map[src]  # read from the cache
-    except KeyError:  # fill the cache
-        splits = split_map[src] = list(sourceconverter.split_source(src))
-        if len(splits) > 1:
-            logging.debug(
-                'Splitting %s "%s" in %d sources', src.__class__.__name__,
-                src.source_id, len(splits))
-    for split in splits:
-        if has_serial:
-            nr = split.num_ruptures
-            split.serial = src.serial[start:start + nr]
-            start += nr
-        yield split
-
-
 def collect_source_model_paths(smlt):
     """
     Given a path to a source model logic tree or a file-like, collect all of
@@ -904,17 +938,15 @@ def collect_source_model_paths(smlt):
 
 class SourceInfo(object):
     dt = numpy.dtype([
-        ('grp_id', numpy.uint32),          # 0
-        ('source_id', (bytes, 100)),       # 1
-        ('source_class', (bytes, 30)),     # 2
-        ('num_ruptures', numpy.uint32),    # 3
-        ('calc_time', numpy.float32),      # 4
-        ('num_sites', numpy.uint32),       # 5
-        ('num_split',  numpy.uint32),      # 6
+        ('source_id', (bytes, 100)),       # 0
+        ('source_class', (bytes, 30)),     # 1
+        ('num_ruptures', numpy.uint32),    # 2
+        ('calc_time', numpy.float32),      # 3
+        ('num_sites', numpy.uint32),       # 4
+        ('num_split',  numpy.uint32),      # 5
     ])
 
     def __init__(self, src, calc_time=0, num_split=0):
-        self.grp_id = src.src_group_id
         self.source_id = src.source_id
         self.source_class = src.__class__.__name__
         self.num_ruptures = src.num_ruptures
