@@ -45,7 +45,7 @@ def getparams(oq):
 
 class ParseConfigTestCase(unittest.TestCase):
 
-    def test_get_oqparam_with_files(self):
+    def test_no_absolute_path(self):
         temp_dir = tempfile.mkdtemp()
         site_model_input = general.writetmp(dir=temp_dir, content="foo")
         job_config = general.writetmp(dir=temp_dir, content="""
@@ -63,6 +63,26 @@ intensity_measure_types = PGA
 investigation_time = 50
 export_dir = %s
         """ % (site_model_input, TMP))
+        with self.assertRaises(ValueError) as ctx:
+            readinput.get_params([job_config])
+        self.assertIn('is an absolute path', str(ctx.exception))
+
+    def test_get_oqparam_with_files(self):
+        temp_dir = tempfile.mkdtemp()
+        site_model_input = general.writetmp(dir=temp_dir, content="foo")
+        job_config = general.writetmp(dir=temp_dir, content="""
+[general]
+calculation_mode = event_based
+[site]
+sites = 0 0
+site_model_file = %s
+maximum_distance=1
+truncation_level=0
+random_seed=0
+intensity_measure_types = PGA
+investigation_time = 50
+export_dir = %s
+        """ % (os.path.basename(site_model_input), TMP))
 
         try:
             exp_base_path = os.path.dirname(job_config)
@@ -92,8 +112,8 @@ export_dir = %s
                 self.assertEqual((job_config, site_model_input), values)
 
                 # checking that warnings work
-                self.assertEqual(warn.call_args[0][0],
-                                 "The parameter 'bar' is unknown, ignoring")
+                self.assertIn('Please remove site_model_file from',
+                              warn.call_args[0][0])
         finally:
             shutil.rmtree(temp_dir)
 
@@ -117,7 +137,7 @@ reference_depth_to_1pt0km_per_sec = 100.0
 intensity_measure_types_and_levels = {'PGA': [0.1, 0.2]}
 investigation_time = 50.
 export_dir = %s
-            """ % (sites_csv, TMP))
+            """ % (os.path.basename(sites_csv), TMP))
             exp_base_path = os.path.dirname(
                 os.path.join(os.path.abspath('.'), source))
 
@@ -165,7 +185,7 @@ reference_depth_to_1pt0km_per_sec = 100.0
 intensity_measure_types_and_levels = {'PGA': [0.1, 0.2]}
 investigation_time = 50.
 export_dir = %s
-""" % (sites_csv, TMP))
+""" % (os.path.basename(sites_csv), TMP))
         oq = readinput.get_oqparam(source, hc_id=1)
         with self.assertRaises(InvalidFile) as ctx:
             readinput.get_mesh(oq)
@@ -379,8 +399,8 @@ class ExposureTestCase(unittest.TestCase):
         exp, _assets = readinput._get_exposure(
             self.exposure, ['structural'], stop='assets')
         self.assertEqual(exp.description, 'Exposure model for buildings')
-        self.assertTrue(exp.insurance_limit_is_absolute)
-        self.assertTrue(exp.deductible_is_absolute)
+        self.assertIsNone(exp.insurance_limit_is_absolute)
+        self.assertIsNone(exp.deductible_is_absolute)
         self.assertEqual([tuple(ct) for ct in exp.cost_types],
                          [('structural', 'per_asset', 'USD')])
 
@@ -470,6 +490,7 @@ POLYGON((68.0 31.5, 69.5 31.5, 69.5 25.5, 68.0 25.5, 68.0 31.5))'''
         oqparam.base_path = '/'
         oqparam.calculation_mode = 'scenario_risk'
         oqparam.all_cost_types = ['structural']
+        oqparam.ignore_missing_costs = []
         oqparam.region_constraint = '''\
 POLYGON((68.0 31.5, 69.5 31.5, 69.5 25.5, 68.0 25.5, 68.0 31.5))'''
         oqparam.inputs = {'exposure': self.exposure2,
