@@ -24,6 +24,7 @@ different kinds of :class:`ground shaking intensity models
 from __future__ import division
 
 import abc
+import sys
 import math
 import warnings
 import functools
@@ -31,12 +32,14 @@ import contextlib
 from scipy.special import ndtr
 import numpy
 
-from openquake.hazardlib import const
-from openquake.hazardlib import imt as imt_module
-from openquake.hazardlib.calc.filters import IntegrationDistance, get_distances
 from openquake.baselib.general import DeprecationWarning, AccumDict
 from openquake.baselib.performance import Monitor
-from openquake.baselib.python3compat import with_metaclass
+from openquake.baselib.python3compat import with_metaclass, raise_
+from openquake.hazardlib import const
+from openquake.hazardlib import imt as imt_module
+from openquake.hazardlib.calc.filters import (
+    IntegrationDistance, get_distances, FarAwayRupture)
+from openquake.hazardlib.probability_map import ProbabilityMap
 
 
 class NonInstantiableError(Exception):
@@ -116,6 +119,7 @@ class ContextMaker(object):
     REQUIRES = ['DISTANCES', 'SITES_PARAMETERS', 'RUPTURE_PARAMETERS']
 
     def __init__(self, gsims, maximum_distance=IntegrationDistance(None)):
+        assert gsims
         self.gsims = gsims
         self.maximum_distance = maximum_distance
         for req in self.REQUIRES:
@@ -182,7 +186,7 @@ class ContextMaker(object):
 
         """
         sctx = SitesContext()
-        sctx.sites = site_collection
+        sctx.sids = site_collection.sids
         for param in self.REQUIRES_SITES_PARAMETERS:
             try:
                 value = getattr(site_collection, param)
@@ -267,6 +271,88 @@ class ContextMaker(object):
         dctx = self.make_distances_context(sites, rupture, {'rjb': distances})
         return (sctx, rctx, dctx)
 
+    def filter_ruptures(self, src, sites):
+        """
+        :param src: a source object, already filtered and split
+        :param sites: a filtered SiteCollection
+        :return: a list of filtered ruptures with context attributes
+        """
+        ruptures = []
+        weight = 1. / (src.num_ruptures or src.count_ruptures())
+        for rup in src.iter_ruptures():
+            rup.weight = weight
+            try:
+                rup.sctx, rup.rctx, rup.dctx = self.make_contexts(sites, rup)
+            except FarAwayRupture:
+                continue
+            ruptures.append(rup)
+        return ruptures
+
+    def make_pmap(self, ruptures, imtls, trunclevel, rup_indep):
+        """
+        :param src: a source object
+        :param ruptures: a list of "dressed" ruptures
+        :param imtls: intensity measure and levels
+        :param trunclevel: truncation level
+        :param rup_indep: True if the ruptures are independent
+        :returns: a ProbabilityMap instance
+        """
+        sids = set()
+        for rup in ruptures:
+            sids.update(rup.sctx.sids)
+        pmap = ProbabilityMap.build(
+            len(imtls.array), len(self.gsims), sids, initvalue=rup_indep)
+        for rup in ruptures:
+            pnes = self._make_pnes(rup, imtls, trunclevel)
+            for sid, pne in zip(rup.sctx.sids, pnes):
+                if rup_indep:
+                    pmap[sid].array *= pne
+                else:
+                    pmap[sid].array += pne * rup.weight
+        tildemap = ~pmap
+        tildemap.eff_ruptures = len(ruptures)
+        return tildemap
+
+    def poe_map(self, src, sites, imtls, trunclevel, ctx_mon, poe_mon,
+                rup_indep=True):
+        """
+        :param src: a source object
+        :param sites: a filtered SiteCollection
+        :param imtls: intensity measure and levels
+        :param trunclevel: truncation level
+        :param ctx_mon: a Monitor instance for make_context
+        :param poe_mon: a Monitor instance for get_poes
+        :param rup_indep: True if the ruptures are independent
+        :returns: a ProbabilityMap instance
+        """
+        with ctx_mon:
+            ruptures = self.filter_ruptures(src, sites)
+        if not ruptures:
+            return {}
+        try:
+            with poe_mon:
+                pmap = self.make_pmap(ruptures, imtls, trunclevel, rup_indep)
+        except Exception as err:
+            etype, err, tb = sys.exc_info()
+            msg = '%s (source id=%s)' % (str(err), src.source_id)
+            raise_(etype, msg, tb)
+        return pmap
+
+    # NB: it is important for this to be fast since it is inside an inner loop
+    def _make_pnes(self, rupture, imtls, trunclevel):
+        pne_array = numpy.zeros(
+            (len(rupture.sctx.sids), len(imtls.array), len(self.gsims)))
+        for i, gsim in enumerate(self.gsims):
+            dctx = rupture.dctx.roundup(gsim.minimum_distance)
+            pnos = []  # list of arrays nsites x nlevels
+            for imt in imtls:
+                poes = gsim.get_poes(
+                    rupture.sctx, rupture.rctx, dctx,
+                    imt_module.from_string(imt), imtls[imt], trunclevel)
+                pnos.append(rupture.get_probability_no_exceedance(poes))
+            pne_array[:, :, i] = numpy.concatenate(pnos, axis=1)
+        return pne_array
+
     def disaggregate(self, sitecol, ruptures, iml4, truncnorm, epsilons,
                      monitor=Monitor()):
         """
@@ -287,14 +373,15 @@ class ContextMaker(object):
         pne_mon = monitor('disaggregate_pne', measuremem=False)
         for rupture in ruptures:
             with ctx_mon:
-                sctx, rctx, dctx = self.make_contexts(
+                sctx, rctx, orig_dctx = self.make_contexts(
                     sitecol, rupture, filter=False)
             if (self.maximum_distance and
-                dctx.rjb.min() > self.maximum_distance(
+                orig_dctx.rjb.min() > self.maximum_distance(
                     rupture.tectonic_region_type, rupture.mag)):
                 continue  # rupture away from all sites
             cache = {}
             for r, gsim in self.gsim_by_rlzi.items():
+                dctx = orig_dctx.roundup(gsim.minimum_distance)
                 for m, imt in enumerate(iml4.imts):
                     for p, poe in enumerate(iml4.poes_disagg):
                         iml = tuple(iml4.array[:, r, m, p])
@@ -408,6 +495,8 @@ class GroundShakingIntensityModel(with_metaclass(MetaGSIM)):
     #: object attributes with same names. Values are in kilometers.
     REQUIRES_DISTANCES = abc.abstractproperty()
 
+    minimum_distance = 0  # can be set by the engine
+
     @abc.abstractmethod
     def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
         """
@@ -807,6 +896,7 @@ class SitesContext(BaseContext):
     Only those required parameters are made available in a result context
     object.
     """
+    # _slots_ is used in hazardlib check_gsim, but not in the engine
     _slots_ = ('vs30', 'vs30measured', 'z1pt0', 'z2pt5', 'backarc',
                'lons', 'lats')
 
@@ -826,6 +916,24 @@ class DistancesContext(BaseContext):
     _slots_ = ('rrup', 'rx', 'rjb', 'rhypo', 'repi', 'ry0', 'rcdpp',
                'azimuth', 'hanging_wall', 'rvolc')
 
+    def roundup(self, minimum_distance):
+        """
+        If the minimum_distance is nonzero, returns a copy of the
+        DistancesContext with updated distances, i.e. the ones below
+        minimum_distance are rounded up to the minimum_distance. Otherwise,
+        returns the original DistancesContext unchanged.
+        """
+        if not minimum_distance:
+            return self
+        ctx = DistancesContext()
+        for dist, array in vars(self).items():
+            small_distances = array < minimum_distance
+            if small_distances.any():
+                array = array[:]  # make a copy first
+                array[small_distances] = minimum_distance
+            setattr(ctx, dist, array)
+        return ctx
+
 
 class RuptureContext(BaseContext):
     """
@@ -1052,5 +1160,4 @@ class CoeffsTable(object):
         min_above = self.sa_coeffs[min_above]
         return dict(
             (co, (min_above[co] - max_below[co]) * ratio + max_below[co])
-            for co in max_below
-        )
+            for co in max_below)
