@@ -4,57 +4,55 @@
 #
 # LICENSE
 #
-# Copyright (c) 2010-2017, GEM Foundation, G. Weatherill, M. Pagani, 
+# Copyright (c) 2010-2017, GEM Foundation, G. Weatherill, M. Pagani,
 # D. Monelli., L. E. Rodriguez-Abreu
 #
-# The Hazard Modeller's Toolkit is free software: you can redistribute 
-# it and/or modify it under the terms of the GNU Affero General Public 
-# License as published by the Free Software Foundation, either version 
+# The Hazard Modeller's Toolkit is free software: you can redistribute
+# it and/or modify it under the terms of the GNU Affero General Public
+# License as published by the Free Software Foundation, either version
 # 3 of the License, or (at your option) any later version.
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with OpenQuake. If not, see <http://www.gnu.org/licenses/>
 #
 # DISCLAIMER
-# 
-# The software Hazard Modeller's Toolkit (openquake.hmtk) provided herein 
-# is released as a prototype implementation on behalf of 
-# scientists and engineers working within the GEM Foundation (Global 
-# Earthquake Model). 
 #
-# It is distributed for the purpose of open collaboration and in the 
+# The software Hazard Modeller's Toolkit (openquake.hmtk) provided herein
+# is released as a prototype implementation on behalf of
+# scientists and engineers working within the GEM Foundation (Global
+# Earthquake Model).
+#
+# It is distributed for the purpose of open collaboration and in the
 # hope that it will be useful to the scientific, engineering, disaster
-# risk and software design communities. 
-# 
-# The software is NOT distributed as part of GEM's OpenQuake suite 
-# (https://www.globalquakemodel.org/tools-products) and must be considered as a 
-# separate entity. The software provided herein is designed and implemented 
-# by scientific staff. It is not developed to the design standards, nor 
-# subject to same level of critical review by professional software 
-# developers, as GEM's OpenQuake software suite.  
-# 
-# Feedback and contribution to the software is welcome, and can be 
-# directed to the hazard scientific staff of the GEM Model Facility 
-# (hazard@globalquakemodel.org). 
-# 
-# The Hazard Modeller's Toolkit (openquake.hmtk) is therefore distributed WITHOUT 
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
-# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
+# risk and software design communities.
+#
+# The software is NOT distributed as part of GEM's OpenQuake suite
+# (https://www.globalquakemodel.org/tools-products) and must be considered as a
+# separate entity. The software provided herein is designed and implemented
+# by scientific staff. It is not developed to the design standards, nor
+# subject to same level of critical review by professional software
+# developers, as GEM's OpenQuake software suite.
+#
+# Feedback and contribution to the software is welcome, and can be
+# directed to the hazard scientific staff of the GEM Model Facility
+# (hazard@globalquakemodel.org).
+#
+# The Hazard Modeller's Toolkit (openquake.hmtk) is therefore distributed
+# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 # for more details.
-# 
-# The GEM Foundation, and the authors of the software, assume no 
-# liability for use of the software. 
-
-#!/usr/bin/env python
+#
+# The GEM Foundation, and the authors of the software, assume no
+# liability for use of the software.
 
 '''
 Module openquake.hmtk.plotting.catalogue.map is a graphical
 function for plotting the spatial distribution of events
 '''
+from builtins import range
 import collections
 import numpy as np
 import matplotlib.pyplot as plt
-from mpl_toolkits.basemap import Basemap
 from matplotlib.colors import Normalize
 from openquake.hmtk.sources.area_source import mtkAreaSource
 from openquake.hmtk.sources.point_source import mtkPointSource
@@ -64,26 +62,24 @@ from openquake.hmtk.sources.simple_fault_source import mtkSimpleFaultSource
 from openquake.hmtk.sources.complex_fault_source import mtkComplexFaultSource
 
 
-DEFAULT_SYMBOLOGY = [(-np.inf, 1., 'k.'), # M < 1
-                     (1., 2., 'g*'), # 1 < M < 2
-                     (2., 3.,'cx'), # 2 < M < 3
-                     (3., 4.,'yd'), # 3 < M < 4
-                     (4., 5.,'m^'), # 4 < M < 5
-                     (5., 6.,'go'), # 5 < M < 6
-                     (6., 7.,'yh'), # 6 < M < 7
-                     (7., 8.,'bs'), # 7 < M < 8
-                     (8., 9.,'k^'), # 8 < M < 9
-                     (9., np.inf,'ro')] # 9 < M < 10
+DEFAULT_SYMBOLOGY = [(-np.inf, 1., 'k.'),  # M < 1
+                     (1., 2., 'g*'),  # 1 < M < 2
+                     (2., 3., 'cx'),  # 2 < M < 3
+                     (3., 4., 'yd'),  # 3 < M < 4
+                     (4., 5., 'm^'),  # 4 < M < 5
+                     (5., 6., 'go'),  # 5 < M < 6
+                     (6., 7., 'yh'),  # 6 < M < 7
+                     (7., 8., 'bs'),  # 7 < M < 8
+                     (8., 9., 'k^'),  # 8 < M < 9
+                     (9., np.inf, 'ro')]  # 9 < M < 10
 
-LEGEND_OFFSET=(1.3, 1.0)
+LEGEND_OFFSET = (1.3, 1.0)
 PORTRAIT_ASPECT = (6, 8)
 LANDSCAPE_ASPECT = (8, 6)
 NCOLS = len(DISSIMILAR_COLOURLIST)
 
-def _fault_polygon_from_mesh(source):
-    """
 
-    """
+def _fault_polygon_from_mesh(source):
     # Mesh
     upper_edge = np.column_stack([source.geometry.mesh.lons[1],
                                   source.geometry.mesh.lats[1],
@@ -93,12 +89,15 @@ def _fault_polygon_from_mesh(source):
                                   source.geometry.mesh.depths[-1]])
     return np.vstack([upper_edge, np.flipud(lower_edge), upper_edge[0, :]])
 
+
 class HMTKBaseMap(object):
     '''
     Class to plot the spatial distribution of events based in the Catalogue
     imported from openquake.hmtk.
     '''
-    def __init__(self, config, title, dpi=300):
+
+    def __init__(self, config,
+                 title=None, dpi=300, ax=None, lat_lon_spacing=2.):
         """
         :param dict config:
             Configuration parameters of the algorithm, containing the
@@ -116,7 +115,9 @@ class HMTKBaseMap(object):
         self.config = config
         self.title = title
         self.dpi = dpi
+        self.lat_lon_spacing = lat_lon_spacing
         self.fig = None
+        self.ax = ax
         self.m = None
         self._build_basemap()
 
@@ -134,7 +135,7 @@ class HMTKBaseMap(object):
         lowcrnrlon = self.config['min_lon']
         uppcrnrlat = self.config['max_lat']
         uppcrnrlon = self.config['max_lon']
-        if not 'resolution' in self.config.keys():
+        if 'resolution' not in self.config.keys():
             self.config['resolution'] = 'l'
 
         lat0 = lowcrnrlat + ((uppcrnrlat - lowcrnrlat) / 2)
@@ -143,22 +144,28 @@ class HMTKBaseMap(object):
             fig_aspect = PORTRAIT_ASPECT
         else:
             fig_aspect = LANDSCAPE_ASPECT
-        self.fig = plt.figure(num=None,
-                              figsize=fig_aspect,
-                              dpi=self.dpi,
-                              facecolor='w',
-                              edgecolor='k')
+
+        if self.ax is None:
+            self.fig, self.ax = plt.subplots(figsize=fig_aspect,
+                                             facecolor='w',
+                                             edgecolor='k')
+        else:
+            self.fig = self.ax.get_figure()
+
         if self.title:
-            plt.title(self.title, fontsize=16)
-        parallels = np.arange(-90., 90., 2.)
-        meridians = np.arange(0., 360., 2.)
+            self.ax.set_title(self.title, fontsize=16)
+        parallels = np.arange(-90., 90., self.lat_lon_spacing)
+        meridians = np.arange(0., 360., self.lat_lon_spacing)
 
         # Build Map
+        # Do not import Basemap at top level since it's an optional feature
+        # and it would break doctests
+        from mpl_toolkits.basemap import Basemap
         self.m = Basemap(
             llcrnrlon=lowcrnrlon, llcrnrlat=lowcrnrlat,
             urcrnrlon=uppcrnrlon, urcrnrlat=uppcrnrlat,
             projection='stere', resolution=self.config['resolution'],
-            area_thresh=1000.0, lat_0=lat0, lon_0=lon0)
+            area_thresh=1000.0, lat_0=lat0, lon_0=lon0, ax=self.ax)
         self.m.drawcountries()
         self.m.drawmapboundary()
         self.m.drawcoastlines()
@@ -198,25 +205,23 @@ class HMTKBaseMap(object):
         # Magnitudes bins and minimum marrker size
         # min_mag = np.min(catalogue.data['magnitude'])
         # max_mag = np.max(catalogue.data['magnitude'])
-        con_min = np.where(np.array([symb[0] for symb in DEFAULT_SYMBOLOGY]) < 
+        con_min = np.where(np.array([symb[0] for symb in DEFAULT_SYMBOLOGY]) <
                            np.min(catalogue.data['magnitude']))[0]
         con_max = np.where(np.array([symb[1] for symb in DEFAULT_SYMBOLOGY]) >
                            np.max(catalogue.data['magnitude']))[0]
         if len(con_min) == 1:
             min_loc = con_min[0]
         else:
-            min_loc = con_min[-1] 
+            min_loc = con_min[-1]
         if len(con_max) == 1:
             max_loc = con_max[0]
         else:
             max_loc = con_max[1]
-        #min_loc = np.where(np.array([symb[0] for symb in DEFAULT_SYMBOLOGY]) <
-        #                   np.min(catalogue.data['magnitude']))[0][-1]
-        #max_loc = np.where(np.array([symb[1] for symb in DEFAULT_SYMBOLOGY]) >
-        #                   np.max(catalogue.data['magnitude']))[0][1]
+        # min_loc = np.where(np.array([symb[0] for symb in DEFAULT_SYMBOLOGY])
+        #                   < np.min(catalogue.data['magnitude']))[0][-1]
+        # max_loc = np.where(np.array([symb[1] for symb in DEFAULT_SYMBOLOGY])
+        #                   > np.max(catalogue.data['magnitude']))[0][1]
         symbology = DEFAULT_SYMBOLOGY[min_loc:max_loc]
-        legend_list = []
-        leg_handles = []
         for sym in symbology:
             # Create legend string
             if np.isinf(sym[0]):
@@ -224,7 +229,7 @@ class HMTKBaseMap(object):
             elif np.isinf(sym[1]):
                 leg_str = 'M >= %5.2f' % sym[0]
             else:
-                leg_str = '%5.2f <= M < %5.2f' %(sym[0], sym[1])
+                leg_str = '%5.2f <= M < %5.2f' % (sym[0], sym[1])
             idx = np.logical_and(catalogue.data['magnitude'] >= sym[0],
                                  catalogue.data['magnitude'] < sym[1])
             mag_size = 1.2 * np.min([sym[0] + 0.5, sym[1] - 0.5])
@@ -232,9 +237,9 @@ class HMTKBaseMap(object):
                           catalogue.data['latitude'][idx])
             self.m.plot(x, y, sym[2], markersize=mag_size, label=leg_str)
 
-        plt.legend(bbox_to_anchor=LEGEND_OFFSET)
+        self.ax.legend(bbox_to_anchor=LEGEND_OFFSET)
         if self.title:
-            plt.title(self.title, fontsize=16)
+            self.ax.set_title(self.title, fontsize=16)
         if not overlay:
             plt.show()
 
@@ -291,7 +296,7 @@ class HMTKBaseMap(object):
         self.m.plot(x, y, border, linewidth=1.3 * border_width)
 
     def _plot_complex_fault(self, source, border='k-', border_width=1.0,
-            min_depth=0., max_depth=None, alpha=1.0):
+                            min_depth=0., max_depth=None, alpha=1.0):
         """
         Plots the simple fault source as a composite of the fault trace
         and the surface projection of the fault.
@@ -310,7 +315,7 @@ class HMTKBaseMap(object):
 
         bottom_edge = np.column_stack([source.geometry.mesh.lons[-1][::-1],
                                        source.geometry.mesh.lats[-1][::-1]])
-        outline = np.vstack([top_edge, bottom_edge, top_edge[0,:]])
+        outline = np.vstack([top_edge, bottom_edge, top_edge[0, :]])
         lons = source.geometry.mesh.lons.flatten()
         lats = source.geometry.mesh.lats.flatten()
         depths = source.geometry.mesh.depths.flatten()
@@ -379,15 +384,15 @@ class HMTKBaseMap(object):
         if not norm:
             norm = Normalize(vmin=np.min(data), vmax=np.max(data))
         x, y, = self.m(longitude, latitude)
-        self.m.scatter(x, y,
-                       marker=shape,
-                       s=size,
-                       c=data,
-                       norm=norm,
-                       alpha=alpha,
-                       linewidths=0.0,
-                       zorder=4)
-        self.m.colorbar()
+        mappable = self.m.scatter(x, y,
+                                  marker=shape,
+                                  s=size,
+                                  c=data,
+                                  norm=norm,
+                                  alpha=alpha,
+                                  linewidths=0.0,
+                                  zorder=4)
+        self.m.colorbar(mappable=mappable, fig=self.fig, ax=self.ax)
         if not overlay:
             plt.show()
 
@@ -421,19 +426,19 @@ class HMTKBaseMap(object):
     def _select_color_mag(self, mag):
         if (mag > 8.0):
             color = 'k'
-            #color.append('k')
+            # color.append('k')
         elif (mag < 8.0) and (mag >= 7.0):
             color = 'b'
-            #color.append('b')
+            # color.append('b')
         elif (mag < 7.0) and (mag >= 6.0):
             color = 'y'
-            #color.append('y')
+            # color.append('y')
         elif (mag < 6.0) and (mag >= 5.0):
             color = 'g'
-            #color.append('g')
+            # color.append('g')
         elif (mag < 5.0):
             color = 'm'
-            #color.append('m')
+            # color.append('m')
         return color
 
     def add_focal_mechanism(self, catalogue, magnitude=None, overlay=True):
@@ -457,23 +462,23 @@ class HMTKBaseMap(object):
                 size = mag * 10000
                 beach = Beach(focal_mechanism, linewidth=1, xy=(x, y),
                               width=size, zorder=size, facecolor=color)
-                plt.gca().add_collection(beach)
+                self.ax.add_collection(beach)
                 if not overlay:
                     plt.show()
         else:
-            for i in xrange(0, catalogue.get_number_tensors()):
+            for i in range(0, catalogue.get_number_tensors()):
                 x, y = self.m(longitude[i], latitude[i])
                 self.m.plot(x, y)
                 focal_mechanism = [strike[i], dip[i], rake[i]]
                 size = magnitude * 10000.
                 beach = Beach(focal_mechanism, linewidth=1, xy=(x, y),
                               width=size, zorder=size, facecolor='r')
-                plt.gca().add_collection(beach)
+                self.ax.add_collection(beach)
                 if not overlay:
                     plt.show()
 
     def add_catalogue_cluster(self, catalogue, vcl, flagvector,
-            cluster_id=None, overlay=True):
+                              cluster_id=None, overlay=True):
         """
         Creates a plot of a catalogue showing where particular clusters exist
         """
